"""
Test breakpoint names.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import TestBase, VALID_BREAKPOINT, VALID_TARGET
from lldbsuite.test import lldbutil


class BreakpointNames(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @add_test_categories(["pyapi"])
    def test_setting_names(self):
        """Use Python APIs to test that we can set breakpoint names."""
        self.build()
        self.setup_target()
        self.do_check_names()

    def test_illegal_names(self):
        """Use Python APIs to test that we don't allow illegal names."""
        self.build()
        self.setup_target()
        self.do_check_illegal_names()

    def test_using_names(self):
        """Use Python APIs to test that operations on names works correctly."""
        self.build()
        self.setup_target()
        self.do_check_using_names()

    def test_configuring_names(self):
        """Use Python APIs to test that configuring options on breakpoint names works correctly."""
        self.build()
        self.make_a_dummy_name()
        self.setup_target()
        self.do_check_configuring_names()

    def test_configuring_permissions_sb(self):
        """Use Python APIs to test that configuring permissions on names works correctly."""
        self.build()
        self.setup_target()
        self.do_check_configuring_permissions_sb()

    def test_configuring_permissions_cli(self):
        """Use Python APIs to test that configuring permissions on names works correctly."""
        self.build()
        self.setup_target()
        self.do_check_configuring_permissions_cli()

    def test_breakpointname_enabled(self):
        """Use Commands and Python APIs to test that enabling and disabling a
        breakpoint name is propagated to all breakpoints with that name.
        """
        self.build()
        self.setup_target()
        self.do_check_breakpointname_enabled()

    def setup_target(self):
        exe = self.getBuildArtifact("a.out")

        # Create a targets we are making breakpoint in and copying to:
        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)
        self.main_file_spec = lldb.SBFileSpec(
            os.path.join(self.getSourceDir(), "main.c")
        )

    def check_name_in_target(self, bkpt_name):
        name_list = lldb.SBStringList()
        self.target.GetBreakpointNames(name_list)
        found_it = False
        for name in name_list:
            if name == bkpt_name:
                found_it = True
                break
        self.assertTrue(
            found_it, "Didn't find the name %s in the target's name list:" % (bkpt_name)
        )

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # These are the settings we're going to be putting into names & breakpoints:
        self.bp_name_string = "ABreakpoint"
        self.is_one_shot = True
        self.ignore_count = 1000
        self.condition = "1 == 2"
        self.auto_continue = True
        self.tid = 0xAAAA
        self.tidx = 10
        self.thread_name = "Fooey"
        self.queue_name = "Blooey"
        self.cmd_list = lldb.SBStringList()
        self.cmd_list.AppendString("frame var")
        self.cmd_list.AppendString("bt")
        self.help_string = "I do something interesting"

    def do_check_names(self):
        """Use Python APIs to check that we can set & retrieve breakpoint names"""
        bkpt = self.target.BreakpointCreateByLocation(self.main_file_spec, 10)
        bkpt_name = "ABreakpoint"
        other_bkpt_name = "_AnotherBreakpoint"

        # Add a name and make sure we match it:
        success = bkpt.AddNameWithErrorHandling(bkpt_name)
        self.assertSuccess(success, "We couldn't add a legal name to a breakpoint.")

        matches = bkpt.MatchesName(bkpt_name)
        self.assertTrue(matches, "We didn't match the name we just set")

        # Make sure we don't match irrelevant names:
        matches = bkpt.MatchesName("NotABreakpoint")
        self.assertTrue(not matches, "We matched a name we didn't set.")

        # Make sure the name is also in the target:
        self.check_name_in_target(bkpt_name)

        # Add another name, make sure that works too:
        bkpt.AddNameWithErrorHandling(other_bkpt_name)

        matches = bkpt.MatchesName(bkpt_name)
        self.assertTrue(
            matches, "Adding a name means we didn't match the name we just set"
        )
        self.check_name_in_target(other_bkpt_name)

        # Remove the name and make sure we no longer match it:
        bkpt.RemoveName(bkpt_name)
        matches = bkpt.MatchesName(bkpt_name)
        self.assertTrue(not matches, "We still match a name after removing it.")

        # Make sure the name list has the remaining name:
        name_list = lldb.SBStringList()
        bkpt.GetNames(name_list)
        num_names = name_list.GetSize()
        self.assertEqual(
            num_names, 1, "Name list has %d items, expected 1." % (num_names)
        )

        name = name_list.GetStringAtIndex(0)
        self.assertEqual(
            name,
            other_bkpt_name,
            "Remaining name was: %s expected %s." % (name, other_bkpt_name),
        )

    def do_check_illegal_names(self):
        """Use Python APIs to check that we reject illegal names."""
        bkpt = self.target.BreakpointCreateByLocation(self.main_file_spec, 10)
        bad_names = [
            "-CantStartWithADash",
            "1CantStartWithANumber",
            "^CantStartWithNonAlpha",
            "CantHave-ADash",
            "Cant Have Spaces",
        ]
        for bad_name in bad_names:
            success = bkpt.AddNameWithErrorHandling(bad_name)
            self.assertTrue(
                success.Fail(), "We allowed an illegal name: %s" % (bad_name)
            )
            bp_name = lldb.SBBreakpointName(self.target, bad_name)
            self.assertFalse(
                bp_name.IsValid(),
                "We made a breakpoint name with an illegal name: %s" % (bad_name),
            )

            retval = lldb.SBCommandReturnObject()
            self.dbg.GetCommandInterpreter().HandleCommand(
                "break set -n whatever -N '%s'" % (bad_name), retval
            )
            self.assertTrue(
                not retval.Succeeded(),
                "break set succeeded with: illegal name: %s" % (bad_name),
            )

    def do_check_using_names(self):
        """Use Python APIs to check names work in place of breakpoint ID's."""

        # Create a dummy breakpoint to use up ID 1
        _ = self.target.BreakpointCreateByLocation(self.main_file_spec, 30)

        # Create a breakpoint to test with
        bkpt = self.target.BreakpointCreateByLocation(self.main_file_spec, 10)
        bkpt_name = "ABreakpoint"
        bkpt_id = bkpt.GetID()
        other_bkpt_name = "_AnotherBreakpoint"

        # Add a name and make sure we match it:
        success = bkpt.AddNameWithErrorHandling(bkpt_name)
        self.assertSuccess(success, "We couldn't add a legal name to a breakpoint.")

        bkpts = lldb.SBBreakpointList(self.target)
        self.target.FindBreakpointsByName(bkpt_name, bkpts)

        self.assertEqual(bkpts.GetSize(), 1, "One breakpoint matched.")
        found_bkpt = bkpts.GetBreakpointAtIndex(0)
        self.assertEqual(bkpt.GetID(), found_bkpt.GetID(), "The right breakpoint.")
        self.assertEqual(bkpt.GetID(), bkpt_id, "With the same ID as before.")
        self.assertEqual(bkpts[0].GetID(), bkpt.GetID(), "subscript [0] matches")
        self.assertEqual(bkpts[-1].GetID(), bkpt.GetID(), "subscript [-1] matches")

        retval = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            "break disable %s" % (bkpt_name), retval
        )
        self.assertTrue(
            retval.Succeeded(), "break disable failed with: %s." % (retval.GetError())
        )
        self.assertTrue(not bkpt.IsEnabled(), "We didn't disable the breakpoint.")

        # Also make sure we don't apply commands to non-matching names:
        self.dbg.GetCommandInterpreter().HandleCommand(
            "break modify --one-shot 1 %s" % (other_bkpt_name), retval
        )
        self.assertTrue(
            retval.Succeeded(), "break modify failed with: %s." % (retval.GetError())
        )
        self.assertTrue(
            not bkpt.IsOneShot(), "We applied one-shot to the wrong breakpoint."
        )

    def check_option_values(self, bp_object):
        self.assertEqual(bp_object.one_shot, self.is_one_shot, "IsOneShot")
        self.assertEqual(bp_object.ignore_count, self.ignore_count, "IgnoreCount")
        self.assertEqual(bp_object.condition, self.condition, "Condition")
        self.assertEqual(bp_object.auto_continue, self.auto_continue, "AutoContinue")
        self.assertEqual(bp_object.thread_id, self.tid, "Thread ID")
        self.assertEqual(bp_object.thread_index, self.tidx, "Thread Index")
        self.assertEqual(bp_object.thread_name, self.thread_name, "Thread Name")
        self.assertEqual(bp_object.queue_name, self.queue_name, "Queue Name")
        set_cmds = lldb.SBStringList()
        bp_object.GetCommandLineCommands(set_cmds)
        self.assertEqual(
            set_cmds.GetSize(), self.cmd_list.GetSize(), "Size of command line commands"
        )
        for idx in range(0, set_cmds.GetSize()):
            self.assertEqual(
                self.cmd_list.GetStringAtIndex(idx),
                set_cmds.GetStringAtIndex(idx),
                "Command %d" % (idx),
            )

    def make_a_dummy_name(self):
        "This makes a breakpoint name in the dummy target to make sure it gets copied over"

        dummy_target = self.dbg.GetDummyTarget()
        self.assertTrue(dummy_target.IsValid(), "Dummy target was not valid.")

        def cleanup():
            self.dbg.GetDummyTarget().DeleteBreakpointName(self.bp_name_string)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Now find it in the dummy target, and make sure these settings took:
        bp_name = lldb.SBBreakpointName(dummy_target, self.bp_name_string)
        # Make sure the name is right:
        self.assertEqual(
            bp_name.GetName(),
            self.bp_name_string,
            "Wrong bp_name: %s" % (bp_name.GetName()),
        )
        bp_name.SetOneShot(self.is_one_shot)
        bp_name.SetIgnoreCount(self.ignore_count)
        bp_name.SetCondition(self.condition)
        bp_name.SetAutoContinue(self.auto_continue)
        bp_name.SetThreadID(self.tid)
        bp_name.SetThreadIndex(self.tidx)
        bp_name.SetThreadName(self.thread_name)
        bp_name.SetQueueName(self.queue_name)
        bp_name.SetCommandLineCommands(self.cmd_list)

        # Now look it up again, and make sure it got set correctly.
        bp_name = lldb.SBBreakpointName(dummy_target, self.bp_name_string)
        self.assertTrue(bp_name.IsValid(), "Failed to make breakpoint name.")
        self.check_option_values(bp_name)

    def do_check_configuring_names(self):
        """Use Python APIs to check that configuring breakpoint names works correctly."""
        other_bp_name_string = "AnotherBreakpointName"
        cl_bp_name_string = "CLBreakpointName"

        # Now find the version copied in from the dummy target, and make sure these settings took:
        bp_name = lldb.SBBreakpointName(self.target, self.bp_name_string)
        self.assertTrue(bp_name.IsValid(), "Failed to make breakpoint name.")
        self.check_option_values(bp_name)

        # Now add this name to a breakpoint, and make sure it gets configured properly
        bkpt = self.target.BreakpointCreateByLocation(self.main_file_spec, 10)
        success = bkpt.AddNameWithErrorHandling(self.bp_name_string)
        self.assertSuccess(success, "Couldn't add this name to the breakpoint")
        self.check_option_values(bkpt)

        # Now make a name from this breakpoint, and make sure the new name is properly configured:
        new_name = lldb.SBBreakpointName(bkpt, other_bp_name_string)
        self.assertTrue(
            new_name.IsValid(), "Couldn't make a valid bp_name from a breakpoint."
        )
        self.check_option_values(bkpt)

        # Now change the name's option and make sure it gets propagated to
        # the breakpoint:
        new_auto_continue = not self.auto_continue
        bp_name.SetAutoContinue(new_auto_continue)
        self.assertEqual(
            bp_name.GetAutoContinue(),
            new_auto_continue,
            "Couldn't change auto-continue on the name",
        )
        self.assertEqual(
            bkpt.GetAutoContinue(),
            new_auto_continue,
            "Option didn't propagate to the breakpoint.",
        )

        # Now make this same breakpoint name - but from the command line
        cmd_str = (
            "breakpoint name configure %s -o %d -i %d -c '%s' -G %d -t %d -x %d -T '%s' -q '%s' -H '%s'"
            % (
                cl_bp_name_string,
                self.is_one_shot,
                self.ignore_count,
                self.condition,
                self.auto_continue,
                self.tid,
                self.tidx,
                self.thread_name,
                self.queue_name,
                self.help_string,
            )
        )
        for cmd in self.cmd_list:
            cmd_str += " -C '%s'" % (cmd)

        self.runCmd(cmd_str, check=True)
        # Now look up this name again and check its options:
        cl_name = lldb.SBBreakpointName(self.target, cl_bp_name_string)
        self.check_option_values(cl_name)
        # Also check the help string:
        self.assertEqual(
            self.help_string, cl_name.GetHelpString(), "Help string didn't match"
        )
        # Change the name and make sure that works:
        new_help = "I do something even more interesting"
        cl_name.SetHelpString(new_help)
        self.assertEqual(new_help, cl_name.GetHelpString(), "SetHelpString didn't")

        # We should have three names now, make sure the target can list them:
        name_list = lldb.SBStringList()
        self.target.GetBreakpointNames(name_list)
        for name_string in [
            self.bp_name_string,
            other_bp_name_string,
            cl_bp_name_string,
        ]:
            self.assertIn(
                name_string, name_list, "Didn't find %s in names" % (name_string)
            )

        # Delete the name from the current target.  Make sure that works and deletes the
        # name from the breakpoint as well:
        self.target.DeleteBreakpointName(self.bp_name_string)
        name_list.Clear()
        self.target.GetBreakpointNames(name_list)
        self.assertNotIn(
            self.bp_name_string,
            name_list,
            "Didn't delete %s from a real target" % (self.bp_name_string),
        )
        # Also make sure the name got removed from breakpoints holding it:
        self.assertFalse(
            bkpt.MatchesName(self.bp_name_string),
            "Didn't remove the name from the breakpoint.",
        )

        # Test that deleting the name we injected into the dummy target works (there's also a
        # cleanup that will do this, but that won't test the result...
        dummy_target = self.dbg.GetDummyTarget()
        dummy_target.DeleteBreakpointName(self.bp_name_string)
        name_list.Clear()
        dummy_target.GetBreakpointNames(name_list)
        self.assertNotIn(
            self.bp_name_string,
            name_list,
            "Didn't delete %s from the dummy target" % (self.bp_name_string),
        )
        # Also make sure the name got removed from breakpoints holding it:
        self.assertFalse(
            bkpt.MatchesName(self.bp_name_string),
            "Didn't remove the name from the breakpoint.",
        )

    def check_permission_results(self, bp_name):
        self.assertFalse(bp_name.GetAllowDelete(), "Didn't set allow delete.")
        protected_bkpt = self.target.BreakpointCreateByLocation(self.main_file_spec, 10)
        protected_id = protected_bkpt.GetID()

        unprotected_bkpt = self.target.BreakpointCreateByLocation(
            self.main_file_spec, 10
        )
        unprotected_id = unprotected_bkpt.GetID()

        success = protected_bkpt.AddNameWithErrorHandling(self.bp_name_string)
        self.assertSuccess(success, "Couldn't add this name to the breakpoint")

        self.target.DisableAllBreakpoints()
        self.assertTrue(
            protected_bkpt.IsEnabled(), "Didnt' keep breakpoint from being disabled"
        )
        self.assertFalse(
            unprotected_bkpt.IsEnabled(),
            "Protected too many breakpoints from disabling.",
        )

        # Try from the command line too:
        unprotected_bkpt.SetEnabled(True)
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("break disable", result)
        self.assertTrue(result.Succeeded())
        self.assertTrue(
            protected_bkpt.IsEnabled(), "Didnt' keep breakpoint from being disabled"
        )
        self.assertFalse(
            unprotected_bkpt.IsEnabled(),
            "Protected too many breakpoints from disabling.",
        )

        self.target.DeleteAllBreakpoints()
        bkpt = self.target.FindBreakpointByID(protected_id)
        self.assertTrue(
            bkpt.IsValid(), "Didn't keep the breakpoint from being deleted."
        )
        bkpt = self.target.FindBreakpointByID(unprotected_id)
        self.assertFalse(
            bkpt.IsValid(), "Protected too many breakpoints from deletion."
        )

        # Remake the unprotected breakpoint and try again from the command line:
        unprotected_bkpt = self.target.BreakpointCreateByLocation(
            self.main_file_spec, 10
        )
        unprotected_id = unprotected_bkpt.GetID()

        self.dbg.GetCommandInterpreter().HandleCommand("break delete -f", result)
        self.assertTrue(result.Succeeded())
        bkpt = self.target.FindBreakpointByID(protected_id)
        self.assertTrue(
            bkpt.IsValid(), "Didn't keep the breakpoint from being deleted."
        )
        bkpt = self.target.FindBreakpointByID(unprotected_id)
        self.assertFalse(
            bkpt.IsValid(), "Protected too many breakpoints from deletion."
        )

    def do_check_configuring_permissions_sb(self):
        bp_name = lldb.SBBreakpointName(self.target, self.bp_name_string)

        # Make a breakpoint name with delete disallowed:
        bp_name = lldb.SBBreakpointName(self.target, self.bp_name_string)
        self.assertTrue(
            bp_name.IsValid(), "Failed to make breakpoint name for valid name."
        )

        bp_name.SetAllowDelete(False)
        bp_name.SetAllowDisable(False)
        bp_name.SetAllowList(False)
        self.check_permission_results(bp_name)

    def do_check_configuring_permissions_cli(self):
        # Make the name with the right options using the command line:
        self.runCmd(
            "breakpoint name configure -L 0 -D 0 -A 0 %s" % (self.bp_name_string),
            check=True,
        )
        # Now look up the breakpoint we made, and check that it works.
        bp_name = lldb.SBBreakpointName(self.target, self.bp_name_string)
        self.assertTrue(
            bp_name.IsValid(), "Didn't make a breakpoint name we could find."
        )
        self.check_permission_results(bp_name)

    def do_check_breakpointname_enabled(self):
        target: lldb.SBTarget = self.target
        self.assertTrue(target.IsValid(), "Target name must be valid.")
        bp_name: lldb.SBBreakpointName = lldb.SBBreakpointName(
            target, self.bp_name_string
        )
        self.assertTrue(bp_name.IsValid(), "Breakpoint name must be valid.")

        # Create two function breakpoints a and b.
        a_breakpoint: lldb.SBBreakpoint = target.BreakpointCreateByName("a")
        self.assertTrue(a_breakpoint and a_breakpoint.IsValid(), VALID_BREAKPOINT)
        self.assertEqual(a_breakpoint.GetNumLocations(), 1)
        self.assertTrue(a_breakpoint.IsEnabled())
        self.assertTrue(a_breakpoint.AddNameWithErrorHandling(self.bp_name_string))

        b_breakpoint = target.BreakpointCreateByName("b")
        self.assertTrue(b_breakpoint and b_breakpoint.IsValid(), VALID_BREAKPOINT)
        self.assertEqual(b_breakpoint.GetNumLocations(), 1)
        self.assertTrue(b_breakpoint.IsEnabled())
        self.assertTrue(b_breakpoint.AddNameWithErrorHandling(self.bp_name_string))

        # enabled and disable the function breakpoints with the breakpoint name.
        # With API.
        bp_name.SetEnabled(False)
        self.assertFalse(bp_name.IsEnabled())
        self.assertFalse(a_breakpoint.IsEnabled())
        self.assertFalse(b_breakpoint.IsEnabled())
        bp_name.SetEnabled(True)
        self.assertTrue(a_breakpoint.IsEnabled())
        self.assertTrue(b_breakpoint.IsEnabled())
        self.assertTrue(bp_name.IsEnabled())

        # With cli.
        self.runCmd(f"breakpoint name configure {self.bp_name_string} --disable")
        self.assertFalse(bp_name.IsEnabled())
        self.assertFalse(a_breakpoint.IsEnabled())
        self.assertFalse(b_breakpoint.IsEnabled())
        self.runCmd(f"breakpoint name configure {self.bp_name_string} --enable")
        self.assertTrue(a_breakpoint.IsEnabled())
        self.assertTrue(b_breakpoint.IsEnabled())
        self.assertTrue(bp_name.IsEnabled())

        # Disabling all the Breakpoints in a BreakpointName
        # does not disable the BreakpointName.
        a_breakpoint.SetEnabled(False)
        b_breakpoint.SetEnabled(False)
        self.assertFalse(a_breakpoint.IsEnabled())
        self.assertFalse(b_breakpoint.IsEnabled())
        self.assertTrue(bp_name.IsEnabled())

        # BreakpointName should enable all disabled breakpoints with the name.
        b_breakpoint.SetEnabled(True)
        bp_name.SetEnabled(True)
        self.assertTrue(a_breakpoint.IsEnabled())
        self.assertTrue(b_breakpoint.IsEnabled())
        self.assertTrue(bp_name.IsEnabled())



"""
Test bt --provider * in a multithreaded scenario with even/odd thread-filtered
providers and an uppercase provider chained on top.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import TestBase
from lldbsuite.test import lldbutil


class FrameProviderThreadFilterTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.cpp"

    def build_and_stop_all_threads(self):
        """Build, set a breakpoint, and continue until all 3 worker threads
        have hit it. Returns (target, process, worker_threads)."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self,
            "break in thread",
            lldb.SBFileSpec(self.source),
            only_one_thread=False,
        )

        # All threads synchronize via an atomic barrier, then each one
        # hits the breakpoint while spinning. Continue until all 3 have hit.
        while bkpt.GetHitCount() < 3:
            process.Continue()

        # After 3 hits, all worker threads are alive in the spin loop.
        # Collect all non-main threads.
        worker_threads = []
        for t in process:
            for i in range(t.GetNumFrames()):
                if "thread_work" in (t.GetFrameAtIndex(i).GetFunctionName() or ""):
                    worker_threads.append(t)
                    break

        self.assertEqual(len(worker_threads), 3, "Expected 3 worker threads")
        return target, process, worker_threads

    def register_providers(self, target):
        """Import the script and register all three providers in order."""
        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        for cls in ("EvenThreadProvider", "OddThreadProvider", "UpperCaseProvider"):
            target.RegisterScriptedFrameProvider(
                "frame_provider." + cls,
                lldb.SBStructuredData(),
                error,
            )
            self.assertTrue(error.Success(), f"Should register {cls}: {error}")

    @skipIf(oslist=["windows"], bugnumber="github.com/llvm/llvm-project/issues/191222")
    def test_bt_provider_star_with_thread_filter(self):
        """
        Register EvenThreadProvider, OddThreadProvider, and UpperCaseProvider.
        For each worker thread, 'bt --provider *' should show:
        - Base Unwinder section with original names (thread_work).
        - UpperCaseProvider section with the fully chained result.
        - The non-applicable prefix provider should NOT appear.
        The default 'bt' should show the final upper-cased + prefixed names.
        """
        target, process, worker_threads = self.build_and_stop_all_threads()
        self.register_providers(target)

        for thread in worker_threads:
            prefix = "EVEN_THREAD_" if thread.GetIndexID() % 2 == 0 else "ODD_THREAD_"
            excluded = (
                "OddThreadProvider"
                if prefix == "EVEN_THREAD_"
                else "EvenThreadProvider"
            )

            process.SetSelectedThread(thread)

            self.expect(
                "bt --provider '*'",
                substrs=["Base Unwinder", "thread_work", "UpperCaseProvider", prefix],
            )
            self.expect(
                "bt --provider '*'",
                matching=False,
                substrs=[excluded],
            )
            self.expect("bt", substrs=[prefix])



import itertools

import gdbremote_testcase
import lldbgdbserverutils
from lldbsuite.support import seven
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class GdbRemoteLaunchTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
    @skipIfWindows  # No pty support to test any inferior output
    @add_test_categories(["llgs"])
    def test_launch_via_A(self):
        self.build()
        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)
        self.do_handshake()
        exe_path = lldbutil.install_to_target(self, self.getBuildArtifact("a.out"))
        args = [exe_path, "stderr:arg1", "stderr:arg2", "stderr:arg3"]
        hex_args = [seven.hexlify(x) for x in args]

        # NB: strictly speaking we should use %x here but this packet
        # is deprecated, so no point in changing lldb-server's expectations
        self.test_sequence.add_log_lines(
            [
                "read packet: $A %d,0,%s,%d,1,%s,%d,2,%s,%d,3,%s#00"
                % tuple(itertools.chain.from_iterable([(len(x), x) for x in hex_args])),
                "send packet: $OK#00",
                "read packet: $c#00",
                "send packet: $W00#00",
            ],
            True,
        )
        context = self.expect_gdbremote_sequence()
        self.assertEqual(context["O_content"], b"arg1\r\narg2\r\narg3\r\n")

    @skipIfWindows  # No pty support to test any inferior output
    @add_test_categories(["llgs"])
    def test_launch_via_vRun(self):
        self.build()
        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)
        self.do_handshake()
        exe_path = lldbutil.install_to_target(self, self.getBuildArtifact("a.out"))
        args = [exe_path, "stderr:arg1", "stderr:arg2", "stderr:arg3"]
        hex_args = [seven.hexlify(x) for x in args]

        self.test_sequence.add_log_lines(
            [
                "read packet: $vRun;%s;%s;%s;%s#00" % tuple(hex_args),
                {"direction": "send", "regex": r"^\$T([0-9a-fA-F]+)"},
                "read packet: $c#00",
                "send packet: $W00#00",
            ],
            True,
        )
        context = self.expect_gdbremote_sequence()
        self.assertEqual(context["O_content"], b"arg1\r\narg2\r\narg3\r\n")

    @add_test_categories(["llgs"])
    @skipIfWindows  # Sometimes returns '$E1f'.
    def test_launch_via_vRun_no_args(self):
        self.build()
        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)
        self.do_handshake()
        exe_path = lldbutil.install_to_target(self, self.getBuildArtifact("a.out"))
        hex_path = seven.hexlify(exe_path)

        self.test_sequence.add_log_lines(
            [
                "read packet: $vRun;%s#00" % (hex_path,),
                {"direction": "send", "regex": r"^\$T([0-9a-fA-F]+)"},
                "read packet: $c#00",
                "send packet: $W00#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @add_test_categories(["llgs"])
    @skipIfRemote
    def test_launch_failure_via_vRun(self):
        self.build()
        exe_path = self.getBuildArtifact("a.out")
        hex_path = seven.hexlify(exe_path)

        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)
        self.do_handshake()
        self.test_sequence.add_log_lines(
            [
                "read packet: $QEnableErrorStrings#00",
                "send packet: $OK#00",
                "read packet: $vRun;%s#00" % hex_path,
                {
                    "direction": "send",
                    "regex": r"^\$E..;([0-9a-fA-F]+)#",
                    "capture": {1: "msg"},
                },
            ],
            True,
        )
        with open(exe_path, "ab") as exe_for_writing:
            context = self.expect_gdbremote_sequence()
        self.assertRegex(
            seven.unhexlify(context.get("msg")),
            r"(execve failed: Text file busy|The process cannot access the file because it is being used by another process.)",
        )

    @skipIfWindows  # No pty support to test any inferior output
    @add_test_categories(["llgs"])
    def test_QEnvironment(self):
        self.build()
        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)
        self.do_handshake()
        exe_path = lldbutil.install_to_target(self, self.getBuildArtifact("a.out"))
        env = {"FOO": "test", "BAR": "a=z"}
        args = [exe_path, "print-env:FOO", "print-env:BAR"]
        hex_args = [seven.hexlify(x) for x in args]

        for key, value in env.items():
            self.test_sequence.add_log_lines(
                [
                    "read packet: $QEnvironment:%s=%s#00" % (key, value),
                    "send packet: $OK#00",
                ],
                True,
            )
        self.test_sequence.add_log_lines(
            [
                "read packet: $vRun;%s#00" % (";".join(hex_args),),
                {"direction": "send", "regex": r"^\$T([0-9a-fA-F]+)"},
                "read packet: $c#00",
                "send packet: $W00#00",
            ],
            True,
        )
        context = self.expect_gdbremote_sequence()
        self.assertEqual(context["O_content"], b"test\r\na=z\r\n")

    @skipIfWindows  # No pty support to test any inferior output
    @add_test_categories(["llgs"])
    def test_QEnvironmentHexEncoded(self):
        self.build()
        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)
        self.do_handshake()
        exe_path = lldbutil.install_to_target(self, self.getBuildArtifact("a.out"))
        env = {"FOO": "test", "BAR": "a=z", "BAZ": "a*}#z"}
        args = [exe_path, "print-env:FOO", "print-env:BAR", "print-env:BAZ"]
        hex_args = [seven.hexlify(x) for x in args]

        for key, value in env.items():
            hex_enc = seven.hexlify("%s=%s" % (key, value))
            self.test_sequence.add_log_lines(
                [
                    "read packet: $QEnvironmentHexEncoded:%s#00" % (hex_enc,),
                    "send packet: $OK#00",
                ],
                True,
            )
        self.test_sequence.add_log_lines(
            [
                "read packet: $vRun;%s#00" % (";".join(hex_args),),
                {"direction": "send", "regex": r"^\$T([0-9a-fA-F]+)"},
                "read packet: $c#00",
                "send packet: $W00#00",
            ],
            True,
        )
        context = self.expect_gdbremote_sequence()
        self.assertEqual(context["O_content"], b"test\r\na=z\r\na*}#z\r\n")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test_event.build_exception import BuildError


class TestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def build_and_run(self, test_file):
        """
        Tries building the given test source and runs to the first breakpoint.
        Returns false if the file fails to build due to an unsupported calling
        convention on the current test target. Returns true if building and
        running to the breakpoint succeeded.
        """
        try:
            self.build(
                dictionary={
                    "C_SOURCES": test_file,
                    "CFLAGS_EXTRAS": "-Werror=ignored-attributes",
                }
            )
        except BuildError as e:
            # Test source failed to build. Check if it failed because the
            # calling convention argument is unsupported/unknown in which case
            # the test should be skipped.
            error_msg = str(e)
            # Clang gives an explicit error when a calling convention is
            # not supported.
            if "calling convention is not supported for this target" in error_msg:
                return False
            # GCC's has two different generic warnings it can emit.
            if "attribute ignored" in error_msg:
                return False
            if "attribute directive ignored " in error_msg:
                return False
            # We got a different build error, so raise it again to fail the
            # test.
            raise
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec(test_file)
        )
        return True

    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_regcall(self):
        if not self.build_and_run("regcall.c"):
            return
        self.expect_expr("func(1, 2, 3, 4)", result_type="int", result_value="10")

    @skipIf(compiler="clang", compiler_version=["<", "17.0"], archs=["arm64"])
    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_ms_abi(self):
        if not self.build_and_run("ms_abi.c"):
            return
        self.expect_expr("func(1, 2, 3, 4)", result_type="int", result_value="10")

    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_stdcall(self):
        if not self.build_and_run("stdcall.c"):
            return
        self.expect_expr("func(1, 2, 3, 4)", result_type="int", result_value="10")

    # Fails on x86, passes elsewhere because clang doesn't support vectorcall on
    # any other architectures.
    @expectedFailureAll(
        triple=re.compile("^(x86|i386)"),
        oslist=["freebsd"], bugnumber="github.com/llvm/llvm-project/issues/56084"
    )
    def test_vectorcall(self):
        if not self.build_and_run("vectorcall.c"):
            return
        self.expect_expr("func(1.0)", result_type="int", result_value="1")

    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_fastcall(self):
        if not self.build_and_run("fastcall.c"):
            return
        self.expect_expr("func(1, 2, 3, 4)", result_type="int", result_value="10")

    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_pascal(self):
        if not self.build_and_run("pascal.c"):
            return
        self.expect_expr("func(1, 2, 3, 4)", result_type="int", result_value="10")

    def test_sysv_abi(self):
        if not self.build_and_run("sysv_abi.c"):
            return
        self.expect_expr("func(1, 2, 3, 4)", result_type="int", result_value="10")



from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from textwrap import dedent
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class MyResponder(MockGDBServerResponder):
    def qXferRead(self, obj, annex, offset, length):
        if annex == "target.xml":
            return (
                dedent(
                    """\
                <?xml version="1.0"?>
                  <target version="1.0">
                    <architecture>aarch64</architecture>
                    <feature name="org.gnu.gdb.aarch64.core">
                      <reg name="x0" bitsize="64"/>
                      <reg name="x1" bitsize="64"/>
                      <reg name="x2" bitsize="64"/>
                      <reg name="x3" bitsize="64"/>
                      <reg name="x4" bitsize="64"/>
                      <reg name="x5" bitsize="64"/>
                      <reg name="x6" bitsize="64"/>
                      <reg name="x7" bitsize="64"/>
                      <reg name="x8" bitsize="64"/>
                      <reg name="x9" bitsize="64"/>
                      <reg name="x10" bitsize="64"/>
                      <reg name="x11" bitsize="64"/>
                      <reg name="x12" bitsize="64"/>
                      <reg name="x13" bitsize="64"/>
                      <reg name="x14" bitsize="64"/>
                      <reg name="x15" bitsize="64"/>
                      <reg name="x16" bitsize="64"/>
                      <reg name="x17" bitsize="64"/>
                      <reg name="x18" bitsize="64"/>
                      <reg name="x19" bitsize="64"/>
                      <reg name="x20" bitsize="64"/>
                      <reg name="x21" bitsize="64"/>
                      <reg name="x22" bitsize="64"/>
                      <reg name="x23" bitsize="64"/>
                      <reg name="x24" bitsize="64"/>
                      <reg name="x25" bitsize="64"/>
                      <reg name="x26" bitsize="64"/>
                      <reg name="x27" bitsize="64"/>
                      <reg name="x28" bitsize="64"/>
                      <reg name="x29" bitsize="64"/>
                      <reg name="x30" bitsize="64"/>
                      <reg name="sp" bitsize="64"/>
                      <reg name="pc" bitsize="64"/>
                    </feature>
                  </target>
                """
                ),
                False,
            )
        else:
            return None, False


class TestQemuAarch64TargetXml(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("AArch64")
    def test_register_augmentation(self):
        """
        Test that we correctly associate the register info with the eh_frame
        register numbers.
        """

        target = self.createTarget("basic_eh_frame-aarch64.yaml")
        self.server.responder = MyResponder()

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )
        self.filecheck("image show-unwind -n foo", __file__, "--check-prefix=UNWIND")


# UNWIND: eh_frame UnwindPlan:
# UNWIND: row[0]:    0: CFA=x29+16 => x30=[CFA-8]



#!/usr/bin/env python
##===-- sandbox.py -------------------------------------------*- Python -*-===##
##
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
##
##===----------------------------------------------------------------------===##


import curses

import os
import signal
import sys

import queue

import cui

event_queue = None


class SandboxUI(cui.CursesUI):
    def __init__(self, screen, event_queue):
        super(SandboxUI, self).__init__(screen, event_queue)

        height, width = self.screen.getmaxyx()
        w2 = width / 2
        h2 = height / 2

        self.wins = []
        # self.wins.append(cui.TitledWin(w2, h2, w2, h2, "Test Window 4"))
        list_win = cui.ListWin(w2, h2, w2, h2)
        for i in range(0, 40):
            list_win.addItem("Item %s" % i)
        self.wins.append(list_win)
        self.wins.append(cui.TitledWin(0, 0, w2, h2, "Test Window 1"))
        self.wins.append(cui.TitledWin(w2, 0, w2, h2, "Test Window 2"))
        self.wins.append(cui.TitledWin(0, h2, w2, h2, "Test Window 3"))

        # def callback(s, content):
        #  self.wins[0].win.scroll(1)
        #  self.wins[0].win.addstr(10, 0, '%s: %s' % (s, content))
        #  self.wins[0].win.scroll(1)
        #  self.el.showPrompt(10, 0)

        # self.wins[0].win.scrollok(1)
        # self.el = cui.CursesEditLine(self.wins[0].win, None,
        #  lambda c: callback('got', c), lambda c: callback('tab', c))
        # self.el.prompt = '>>> '
        # self.el.showPrompt(10, 0)

    def handleEvent(self, event):
        if isinstance(event, int):
            if event == ord("q"):
                sys.exit(0)
            # self.el.handleEvent(event)
        super(SandboxUI, self).handleEvent(event)


def main(screen):
    global event_queue
    event_queue = queue.Queue()

    sandbox = SandboxUI(screen, event_queue)
    sandbox.eventLoop()


if __name__ == "__main__":
    try:
        curses.wrapper(main)
    except KeyboardInterrupt:
        exit()



"""
Test the lldb command line completion mechanism for the 'expr' command.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbplatform
from lldbsuite.test import lldbutil


class CommandLineExprCompletionTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_expr_completion(self):
        self.build()
        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        self.createTestTarget()

        # Try the completion before we have a context to complete on.
        self.assume_no_completions("expr some_expr")
        self.assume_no_completions("expr ")
        self.assume_no_completions("expr f")

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Break here", self.main_source_spec
        )

        # Completing member functions
        self.complete_from_to(
            "expr some_expr.FooNoArgs", "expr some_expr.FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr some_expr.FooWithArgs", "expr some_expr.FooWithArgsBar("
        )
        self.complete_from_to(
            "expr some_expr.FooWithMultipleArgs",
            "expr some_expr.FooWithMultipleArgsBar(",
        )
        self.complete_from_to(
            "expr some_expr.FooUnderscore", "expr some_expr.FooUnderscoreBar_()"
        )
        self.complete_from_to(
            "expr some_expr.FooNumbers", "expr some_expr.FooNumbersBar1()"
        )
        self.complete_from_to(
            "expr some_expr.StaticMemberMethod",
            "expr some_expr.StaticMemberMethodBar()",
        )

        # Completing static functions
        self.complete_from_to(
            "expr Expr::StaticMemberMethod", "expr Expr::StaticMemberMethodBar()"
        )

        # Completing member variables
        self.complete_from_to(
            "expr some_expr.MemberVariab", "expr some_expr.MemberVariableBar"
        )

        # Multiple completions
        self.completions_contain(
            "expr some_expr.",
            [
                "some_expr.FooNumbersBar1()",
                "some_expr.FooUnderscoreBar_()",
                "some_expr.FooWithArgsBar(",
                "some_expr.MemberVariableBar",
            ],
        )

        self.completions_contain(
            "expr some_expr.Foo",
            [
                "some_expr.FooNumbersBar1()",
                "some_expr.FooUnderscoreBar_()",
                "some_expr.FooWithArgsBar(",
            ],
        )

        self.completions_contain(
            "expr ", ["static_cast", "reinterpret_cast", "dynamic_cast"]
        )

        self.completions_contain(
            "expr 1 + ", ["static_cast", "reinterpret_cast", "dynamic_cast"]
        )

        # Completion expr without spaces
        # This is a bit awkward looking for the user, but that's how
        # the completion API works at the moment.
        self.completions_contain("expr 1+", ["1+some_expr", "1+static_cast"])

        # Test with spaces
        self.complete_from_to(
            "expr   some_expr .FooNoArgs", "expr   some_expr .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr  some_expr .FooNoArgs", "expr  some_expr .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr some_expr .FooNoArgs", "expr some_expr .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr some_expr. FooNoArgs", "expr some_expr. FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr some_expr . FooNoArgs", "expr some_expr . FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr Expr :: StaticMemberMethod", "expr Expr :: StaticMemberMethodBar()"
        )
        self.complete_from_to(
            "expr Expr ::StaticMemberMethod", "expr Expr ::StaticMemberMethodBar()"
        )
        self.complete_from_to(
            "expr Expr:: StaticMemberMethod", "expr Expr:: StaticMemberMethodBar()"
        )

        # Test that string literals don't break our parsing logic.
        self.complete_from_to(
            'expr const char *cstr = "some_e"; char c = *cst',
            'expr const char *cstr = "some_e"; char c = *cstr',
        )
        self.complete_from_to(
            'expr const char *cstr = "some_e" ; char c = *cst',
            'expr const char *cstr = "some_e" ; char c = *cstr',
        )
        # Requesting completions inside an incomplete string doesn't provide any
        # completions.
        self.complete_from_to(
            'expr const char *cstr = "some_e', 'expr const char *cstr = "some_e'
        )

        # Completing inside double dash should do nothing
        self.assume_no_completions("expr -i0 -- some_expr.", 10)
        self.assume_no_completions("expr -i0 -- some_expr.", 11)

        # Test with expr arguments
        self.complete_from_to(
            "expr -i0 -- some_expr .FooNoArgs", "expr -i0 -- some_expr .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr  -i0 -- some_expr .FooNoArgs",
            "expr  -i0 -- some_expr .FooNoArgsBar()",
        )

        # Addrof and deref
        self.complete_from_to(
            "expr (*(&some_expr)).FooNoArgs", "expr (*(&some_expr)).FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr (*(&some_expr)) .FooNoArgs", "expr (*(&some_expr)) .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr (* (&some_expr)) .FooNoArgs", "expr (* (&some_expr)) .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr (* (& some_expr)) .FooNoArgs",
            "expr (* (& some_expr)) .FooNoArgsBar()",
        )

        # Addrof and deref (part 2)
        self.complete_from_to(
            "expr (&some_expr)->FooNoArgs", "expr (&some_expr)->FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr (&some_expr) ->FooNoArgs", "expr (&some_expr) ->FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr (&some_expr) -> FooNoArgs", "expr (&some_expr) -> FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr (&some_expr)-> FooNoArgs", "expr (&some_expr)-> FooNoArgsBar()"
        )

        # Builtin arg
        self.complete_from_to("expr static_ca", "expr static_cast")

        # From other files
        self.complete_from_to(
            "expr fwd_decl_ptr->Hidden", "expr fwd_decl_ptr->HiddenMember"
        )

        # Types
        self.complete_from_to("expr LongClassNa", "expr LongClassName")
        self.complete_from_to(
            "expr LongNamespaceName::NestedCla", "expr LongNamespaceName::NestedClass"
        )

        # Namespaces
        self.complete_from_to("expr LongNamespaceNa", "expr LongNamespaceName::")

        # Multiple arguments
        self.complete_from_to(
            "expr &some_expr + &some_e", "expr &some_expr + &some_expr"
        )
        self.complete_from_to(
            "expr SomeLongVarNameWithCapitals + SomeLongVarName",
            "expr SomeLongVarNameWithCapitals + SomeLongVarNameWithCapitals",
        )
        self.complete_from_to(
            "expr SomeIntVar + SomeIntV", "expr SomeIntVar + SomeIntVar"
        )

        # Multiple statements
        self.complete_from_to(
            "expr long LocalVariable = 0; LocalVaria",
            "expr long LocalVariable = 0; LocalVariable",
        )

        # Custom Decls
        self.complete_from_to(
            "expr auto l = [](int LeftHandSide, int bx){ return LeftHandS",
            "expr auto l = [](int LeftHandSide, int bx){ return LeftHandSide",
        )
        self.complete_from_to(
            "expr struct LocalStruct { long MemberName; } ; LocalStruct S; S.Mem",
            "expr struct LocalStruct { long MemberName; } ; LocalStruct S; S.MemberName",
        )

        # Completing function call arguments
        self.complete_from_to(
            "expr some_expr.FooWithArgsBar(some_exp",
            "expr some_expr.FooWithArgsBar(some_expr",
        )
        self.complete_from_to(
            "expr some_expr.FooWithArgsBar(SomeIntV",
            "expr some_expr.FooWithArgsBar(SomeIntVar",
        )
        self.complete_from_to(
            "expr some_expr.FooWithMultipleArgsBar(SomeIntVar, SomeIntVa",
            "expr some_expr.FooWithMultipleArgsBar(SomeIntVar, SomeIntVar",
        )

        # Function return values
        self.complete_from_to(
            "expr some_expr.Self().FooNoArgs", "expr some_expr.Self().FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr some_expr.Self() .FooNoArgs", "expr some_expr.Self() .FooNoArgsBar()"
        )
        self.complete_from_to(
            "expr some_expr.Self(). FooNoArgs", "expr some_expr.Self(). FooNoArgsBar()"
        )

        self.complete_from_to("expr myVec.__f", "expr myVec.__func()")
        self.complete_from_to("expr myVec._F", "expr myVec._Func()")
        self.complete_from_to("expr myVec.__m", "expr myVec.__mem")
        self.complete_from_to("expr myVec._M", "expr myVec._Mem")

    def test_expr_completion_with_descriptions(self):
        self.build()
        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        self.createTestTarget()

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Break here", self.main_source_spec
        )

        self.check_completion_with_desc(
            "expr ",
            [
                # builtin types have no description.
                ["int", ""],
                ["float", ""],
                # VarDecls have their type as description.
                ["some_expr", "Expr &"],
            ],
            enforce_order=True,
        )
        self.check_completion_with_desc(
            "expr some_expr.",
            [
                # Functions have their signature as description.
                ["some_expr.~Expr()", "inline ~Expr()"],
                ["some_expr.operator=(", "inline Expr &operator=(const Expr &)"],
                # FieldDecls have their type as description.
                ["some_expr.MemberVariableBar", "int"],
                [
                    "some_expr.StaticMemberMethodBar()",
                    "static int StaticMemberMethodBar()",
                ],
                ["some_expr.Self()", "Expr &Self()"],
                ["some_expr.FooNoArgsBar()", "int FooNoArgsBar()"],
                ["some_expr.FooWithArgsBar(", "int FooWithArgsBar(int)"],
                ["some_expr.FooNumbersBar1()", "int FooNumbersBar1()"],
                ["some_expr.FooUnderscoreBar_()", "int FooUnderscoreBar_()"],
                [
                    "some_expr.FooWithMultipleArgsBar(",
                    "int FooWithMultipleArgsBar(int, int)",
                ],
            ],
            enforce_order=True,
        )

    def test_expr_completion_max_results(self):
        self.build()
        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        self.createTestTarget()

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Break here", self.main_source_spec
        )

        expected_completions = [
            "some_expr.~Expr()",
            "some_expr.operator=(",  # Copy operator
            "some_expr.operator=(",  # Move operator
            "some_expr.MemberVariableBar",
            "some_expr.StaticMemberMethodBar()",
            "some_expr.Self()",
            "some_expr.FooNoArgsBar()",
            "some_expr.FooWithArgsBar(",
            "some_expr.FooNumbersBar1()",
            "some_expr.FooUnderscoreBar_()",
            "some_expr.FooWithMultipleArgsBar(",
        ]

        for i in range(1, len(expected_completions)):
            self.completions_match(
                "expr some_expr.",
                expected_completions[:i],
                max_completions=i,
            )

    def assume_no_completions(self, str_input, cursor_pos=None):
        interp = self.dbg.GetCommandInterpreter()
        match_strings = lldb.SBStringList()
        if cursor_pos is None:
            cursor_pos = len(str_input)
        num_matches = interp.HandleCompletion(
            str_input, cursor_pos, 0, -1, match_strings
        )

        available_completions = []
        for m in match_strings:
            available_completions.append(m)

        self.assertEqual(
            num_matches,
            0,
            "Got matches, but didn't expect any: " + str(available_completions),
        )

    def completions_contain(self, str_input, items):
        interp = self.dbg.GetCommandInterpreter()
        match_strings = lldb.SBStringList()
        num_matches = interp.HandleCompletion(
            str_input, len(str_input), 0, -1, match_strings
        )
        common_match = match_strings.GetStringAtIndex(0)

        for item in items:
            found = False
            for m in match_strings:
                if m == item:
                    found = True
            if not found:
                # Transform match_strings to a python list with strings
                available_completions = []
                for m in match_strings:
                    available_completions.append(m)
                self.assertTrue(
                    found,
                    "Couldn't find completion "
                    + item
                    + " in completions "
                    + str(available_completions),
                )



"""
Test basics of mini dump debugging.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class MiniDumpTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_process_info_in_mini_dump(self):
        """Test that lldb can read the process information from the minidump."""
        # target create -c fizzbuzz_no_heap.dmp
        self.dbg.CreateTarget("")
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore("fizzbuzz_no_heap.dmp")
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertEqual(self.process.GetNumThreads(), 1)
        self.assertEqual(self.process.GetProcessID(), 4440)

    def test_thread_info_in_mini_dump(self):
        """Test that lldb can read the thread information from the minidump."""
        # target create -c fizzbuzz_no_heap.dmp
        self.dbg.CreateTarget("")
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore("fizzbuzz_no_heap.dmp")
        # This process crashed due to an access violation (0xc0000005) in its
        # one and only thread.
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)
        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonException)
        stop_description = thread.stop_description
        self.assertIn("0xc0000005", stop_description)

    def test_modules_in_mini_dump(self):
        """Test that lldb can read the list of modules from the minidump."""
        # target create -c fizzbuzz_no_heap.dmp
        self.dbg.CreateTarget("")
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore("fizzbuzz_no_heap.dmp")
        self.assertTrue(self.process, PROCESS_IS_VALID)
        expected_modules = [
            {
                "filename": r"C:\Users\amccarth\Documents\Visual Studio 2013\Projects\fizzbuzz\Debug\fizzbuzz.exe",
                "uuid": "0F45B791-9A96-46F9-BF8F-2D6076EA421A-00000011",
            },
            {
                "filename": r"C:\Windows\SysWOW64\ntdll.dll",
                "uuid": "BBB0846A-402C-4052-A16B-67650BBFE6B0-00000002",
            },
            {
                "filename": r"C:\Windows\SysWOW64\kernel32.dll",
                "uuid": "E5CB7E1B-005E-4113-AB98-98D6913B52D8-00000002",
            },
            {
                "filename": r"C:\Windows\SysWOW64\KERNELBASE.dll",
                "uuid": "0BF95241-CB0D-4BD4-AC5D-186A6452E522-00000001",
            },
            {
                "filename": r"C:\Windows\System32\MSVCP120D.dll",
                "uuid": "3C05516E-57E7-40EB-8D3F-9722C5BD80DD-00000001",
            },
            {
                "filename": r"C:\Windows\System32\MSVCR120D.dll",
                "uuid": "6382FB86-46C4-4046-AE42-8D97B3F91FF2-00000001",
            },
        ]
        self.assertEqual(self.target.GetNumModules(), len(expected_modules))
        for module, expected in zip(self.target.modules, expected_modules):
            self.assertTrue(module.IsValid())
            self.assertEqual(module.file.fullpath, expected["filename"])
            self.assertEqual(module.GetUUIDString(), expected["uuid"])

    def test_breakpad_uuid_matching(self):
        """Test that the uuid computation algorithms in minidump and breakpad
        files match."""
        self.target = self.dbg.CreateTarget("")
        self.process = self.target.LoadCore("fizzbuzz_no_heap.dmp")
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.expect(
            "target symbols add fizzbuzz.syms",
            substrs=[
                "symbol file",
                "fizzbuzz.syms",
                "has been added to",
                "fizzbuzz.exe",
            ],
        ),
        self.assertTrue(self.target.modules[0].FindSymbol("main"))

    @skipIfLLVMTargetMissing("X86")
    def test_stack_info_in_mini_dump(self):
        """Test that we can see a trivial stack in a VS-generate mini dump."""
        # target create -c fizzbuzz_no_heap.dmp
        self.dbg.CreateTarget("")
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore("fizzbuzz_no_heap.dmp")
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)

        pc_list = [
            0x00164D14,
            0x00167C79,
            0x00167E6D,
            0x7510336A,
            0x77759882,
            0x77759855,
        ]

        self.assertEqual(thread.GetNumFrames(), len(pc_list))
        for i in range(len(pc_list)):
            frame = thread.GetFrameAtIndex(i)
            self.assertTrue(frame.IsValid())
            self.assertEqual(frame.GetPC(), pc_list[i])
            self.assertTrue(frame.GetModule().IsValid())

    @skipUnlessWindows  # Minidump saving works only on windows
    def test_deeper_stack_in_mini_dump(self):
        """Test that we can examine a more interesting stack in a mini dump."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        core = self.getBuildArtifact("core.dmp")
        try:
            # Set a breakpoint and capture a mini dump.
            target = self.dbg.CreateTarget(exe)
            breakpoint = target.BreakpointCreateByName("bar")
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)
            self.assertTrue(process.SaveCore(core))
            self.assertTrue(os.path.isfile(core))
            self.assertSuccess(process.Kill())

            # Launch with the mini dump, and inspect the stack.
            target = self.dbg.CreateTarget(None)
            process = target.LoadCore(core)
            thread = process.GetThreadAtIndex(0)

            expected_stack = {0: "bar", 1: "foo", 2: "main"}
            self.assertGreaterEqual(thread.GetNumFrames(), len(expected_stack))
            for index, name in expected_stack.items():
                frame = thread.GetFrameAtIndex(index)
                self.assertTrue(frame.IsValid())
                function_name = frame.GetFunctionName()
                self.assertIn(name, function_name)

        finally:
            # Clean up the mini dump file.
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(core):
                os.unlink(core)

    @skipUnlessWindows  # Minidump saving works only on windows
    def test_local_variables_in_mini_dump(self):
        """Test that we can examine local variables in a mini dump."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        core = self.getBuildArtifact("core.dmp")
        try:
            # Set a breakpoint and capture a mini dump.
            target = self.dbg.CreateTarget(exe)
            breakpoint = target.BreakpointCreateByName("bar")
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)
            self.assertTrue(process.SaveCore(core))
            self.assertTrue(os.path.isfile(core))
            self.assertSuccess(process.Kill())

            # Launch with the mini dump, and inspect a local variable.
            target = self.dbg.CreateTarget(None)
            process = target.LoadCore(core)
            thread = process.GetThreadAtIndex(0)
            frame = thread.GetFrameAtIndex(0)
            value = frame.EvaluateExpression("x")
            self.assertEqual(value.GetValueAsSigned(), 3)

        finally:
            # Clean up the mini dump file.
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(core):
                os.unlink(core)



"""
Make sure 'frame var' using DIL parser/evaluator works for namespaces.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil

import os
import shutil
import time


class TestFrameVarDILQualifiedId(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfWindows
    def test_frame_var(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")
        self.expect_var_path("::ns::i", value="1")
        self.expect_var_path("ns::i", value="1")
        self.expect_var_path("::ns::ns::i", value="2")
        self.expect_var_path("ns::ns::i", value="2")

        self.expect_var_path("foo", value="1")
        self.expect_var_path("::(anonymous namespace)::foo", value="13")
        self.expect_var_path("(anonymous namespace)::foo", value="13")
        self.expect_var_path("ns1::(anonymous namespace)::foo", value="5")
        self.expect_var_path(
            "(anonymous namespace)::ns2::(anonymous namespace)::foo",
            value="7",
        )
        self.expect_var_path("::ns1::(anonymous namespace)::foo", value="5")
        self.expect_var_path(
            "::(anonymous namespace)::ns2::(anonymous namespace)::foo",
            value="7",
        )



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GenericUnorderedDataFormatterTestCase(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        TestBase.setUp(self)
        self.namespace = "std"

    def do_test_with_run_command(self):
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_source_regexp(self, "Set break point at this line.")

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)
            self.runCmd("settings set auto-one-line-summaries true", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        ns = self.namespace

        # We check here that the map shows 0 children even with corrupt data.
        self.look_for_content_and_continue(
            "corrupt_map", ["%s::unordered_map" % ns, "size=0 {}"]
        )

        # Ensure key/value children, not wrapped in a layer.
        # This regex depends on auto-one-line-summaries.
        self.runCmd("settings set auto-one-line-summaries false")
        children_are_key_value = r"\[0\] = \{\s*first = "

        unordered_map_type = (
            "std::unordered_map<int, std::basic_string<char, std::char_traits<char>, std::allocator<char>>, std::hash<int>, std::equal_to<int>, std::allocator<std::pair<int const, std::basic_string<char, std::char_traits<char>, std::allocator<char>>>>>"
            if self.getDebugInfo() == "pdb"
            else "UnorderedMap"
        )
        self.look_for_content_and_continue(
            "map",
            [
                unordered_map_type,
                children_are_key_value,
                "size=5 {",
                "hello",
                "world",
                "this",
                "is",
                "me",
            ],
        )

        unordered_mmap_type = (
            "std::unordered_multimap<int, std::basic_string<char, std::char_traits<char>, std::allocator<char>>, std::hash<int>, std::equal_to<int>, std::allocator<std::pair<int const, std::basic_string<char, std::char_traits<char>, std::allocator<char>>>>>"
            if self.getDebugInfo() == "pdb"
            else "UnorderedMultiMap"
        )
        self.look_for_content_and_continue(
            "mmap",
            [
                unordered_mmap_type,
                children_are_key_value,
                "size=6 {",
                "first = 3",
                'second = "this"',
                "first = 2",
                'second = "hello"',
            ],
        )

        ints_unordered_set = (
            "std::unordered_set<int, std::hash<int>, std::equal_to<int>, std::allocator<int>>"
            if self.getDebugInfo() == "pdb"
            else "IntsUnorderedSet"
        )
        self.look_for_content_and_continue(
            "iset",
            [
                ints_unordered_set,
                "size=5 {",
                r"\[\d\] = 5",
                r"\[\d\] = 3",
                r"\[\d\] = 2",
            ],
        )

        strings_unordered_set = (
            "std::unordered_set<std::basic_string<char, std::char_traits<char>, std::allocator<char>>, std::hash<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>, std::equal_to<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>, std::allocator<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>>"
            if self.getDebugInfo() == "pdb"
            else "StringsUnorderedSet"
        )
        self.look_for_content_and_continue(
            "sset",
            [
                strings_unordered_set,
                "size=5 {",
                r'\[\d\] = "is"',
                r'\[\d\] = "world"',
                r'\[\d\] = "hello"',
            ],
        )

        ints_unordered_mset = (
            "std::unordered_multiset<int, std::hash<int>, std::equal_to<int>, std::allocator<int>>"
            if self.getDebugInfo() == "pdb"
            else "IntsUnorderedMultiSet"
        )
        self.look_for_content_and_continue(
            "imset",
            [
                ints_unordered_mset,
                "size=6 {",
                "(\\[\\d\\] = 3(\\n|.)+){3}",
                r"\[\d\] = 2",
                r"\[\d\] = 1",
            ],
        )

        strings_unordered_mset = (
            "std::unordered_multiset<std::basic_string<char, std::char_traits<char>, std::allocator<char>>, std::hash<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>, std::equal_to<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>, std::allocator<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>>"
            if self.getDebugInfo() == "pdb"
            else "StringsUnorderedMultiSet"
        )
        self.look_for_content_and_continue(
            "smset",
            [
                strings_unordered_mset,
                "size=5 {",
                '(\\[\\d\\] = "is"(\\n|.)+){2}',
                '(\\[\\d\\] = "world"(\\n|.)+){2}',
            ],
        )

    def look_for_content_and_continue(self, var_name, patterns):
        self.expect(("frame variable %s" % var_name), ordered=False, patterns=patterns)
        self.expect(("frame variable %s" % var_name), ordered=False, patterns=patterns)
        self.runCmd("continue")

    @add_test_categories(["libstdcxx"])
    def test_with_run_command_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["libstdcxx"])
    def test_with_run_command_libstdcxx_debug(self):
        self.build(
            dictionary={"USE_LIBSTDCPP": 1, "CXXFLAGS_EXTRAS": "-D_GLIBCXX_DEBUG"}
        )
        self.do_test_with_run_command()

    @add_test_categories(["libc++"])
    def test_with_run_command_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["msvcstl"])
    def test_with_run_command_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test_with_run_command()



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    def test(self):
        self.build()
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("lib.cpp")
        )

        frame = thread.selected_frame
        self.assertEqual(frame.GuessLanguage(), lldb.eLanguageTypeC_plus_plus_11)
        self.assertEqual(frame.name, "f()")

        # Test `help`.
        self.expect(
            "help demangle",
            substrs=[
                "Demangle a C++ mangled name.",
                "Syntax: language cplusplus demangle [<mangled-name> ...]",
            ],
        )

        # Run a `language cplusplus` command.
        self.expect("demangle _Z1fv", startstr="_Z1fv ---> f()")
        # Test prefix matching.
        self.expect("dem _Z1fv", startstr="_Z1fv ---> f()")

        # Select the objc caller.
        self.runCmd("up")
        frame = thread.selected_frame
        self.assertEqual(frame.GuessLanguage(), lldb.eLanguageTypeObjC_plus_plus)
        self.assertEqual(frame.name, "main")

        # Ensure `demangle` doesn't resolve from the objc frame.
        self.expect("help demangle", error=True)

        # Run a `language objc` command.
        self.expect(
            "tagged-pointer",
            substrs=[
                "Commands for operating on Objective-C tagged pointers.",
                "Syntax: tagged-pointer <subcommand> [<subcommand-options>]",
                "The following subcommands are supported:",
                "info -- Dump information on a tagged pointer.",
            ],
        )

        # To ensure compatability with existing scripts, a language specific
        # command must not be invoked if another command (such as a python
        # command) has the language specific command name as its prefix.
        #
        # For example, this test loads a `tagged-pointer-collision` command. A
        # script could exist that invokes this command using its prefix
        # `tagged-pointer`, under the assumption that "tagged-pointer" uniquely
        # identifies the python command `tagged-pointer-collision`.
        self.runCmd("command script import commands.py")
        self.expect("tagged-pointer", startstr="ran tagged-pointer-collision")



"""
Use lldb Python SBtarget.WatchpointCreateByAddress() API to create a watchpoint for write of '*g_char_ptr'.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TargetWatchpointCreateByAddressPITestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.cpp"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")
        # This is for verifying that watch location works.
        self.violating_func = "do_bad_thing_with_location"

    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/144777",
    )
    def test_watch_create_by_address(self):
        """Exercise SBTarget.WatchpointCreateByAddress() API to set a watchpoint."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        value = frame0.FindValue("g_char_ptr", lldb.eValueTypeVariableGlobal)
        pointee = value.CreateValueFromAddress(
            "pointee", value.GetValueAsUnsigned(0), value.GetType().GetPointeeType()
        )
        # Watch for write to *g_char_ptr.
        error = lldb.SBError()
        wp_opts = lldb.SBWatchpointOptions()
        wp_opts.SetWatchpointTypeWrite(lldb.eWatchpointWriteTypeOnModify)
        watchpoint = target.WatchpointCreateByAddress(
            value.GetValueAsUnsigned(), 1, wp_opts, error
        )
        self.assertTrue(
            value and watchpoint, "Successfully found the pointer and set a watchpoint"
        )
        self.DebugSBValue(value)
        self.DebugSBValue(pointee)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print(watchpoint)

        # Continue.  Expect the program to stop due to the variable being
        # written to.
        process.Continue()

        if self.TraceOn():
            lldbutil.print_stacktraces(process)

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonWatchpoint)
        self.assertTrue(thread, "The thread stopped due to watchpoint")
        self.DebugSBValue(value)
        self.DebugSBValue(pointee)

        self.expect(
            lldbutil.print_stacktrace(thread, string_buffer=True),
            exe=False,
            substrs=[self.violating_func],
        )

        # This finishes our test.

    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/144777",
    )
    def test_watch_address(self):
        """Exercise SBTarget.WatchAddress() API to set a watchpoint.
        Same as test_watch_create_by_address, but uses the simpler API.
        """
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        value = frame0.FindValue("g_char_ptr", lldb.eValueTypeVariableGlobal)
        pointee = value.CreateValueFromAddress(
            "pointee", value.GetValueAsUnsigned(0), value.GetType().GetPointeeType()
        )
        # Watch for write to *g_char_ptr.
        error = lldb.SBError()
        watch_read = False
        watch_write = True
        watchpoint = target.WatchAddress(
            value.GetValueAsUnsigned(), 1, watch_read, watch_write, error
        )
        self.assertTrue(
            value and watchpoint, "Successfully found the pointer and set a watchpoint"
        )
        self.DebugSBValue(value)
        self.DebugSBValue(pointee)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print(watchpoint)

        # Continue.  Expect the program to stop due to the variable being
        # written to.
        process.Continue()

        if self.TraceOn():
            lldbutil.print_stacktraces(process)

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonWatchpoint)
        self.assertTrue(thread, "The thread stopped due to watchpoint")
        self.DebugSBValue(value)
        self.DebugSBValue(pointee)

        self.expect(
            lldbutil.print_stacktrace(thread, string_buffer=True),
            exe=False,
            substrs=[self.violating_func],
        )

        # This finishes our test.

    # No size constraint on MIPS for watches
    @skipIf(archs=["mips", "mipsel", "mips64", "mips64el"])
    @skipIf(archs=["s390x"])  # Likewise on SystemZ
    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/142196",
    )
    def test_watch_address_with_invalid_watch_size(self):
        """Exercise SBTarget.WatchpointCreateByAddress() API but pass an invalid watch_size."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        value = frame0.FindValue("g_char_ptr", lldb.eValueTypeVariableGlobal)
        pointee = value.CreateValueFromAddress(
            "pointee", value.GetValueAsUnsigned(0), value.GetType().GetPointeeType()
        )

        # debugserver on Darwin AArch64 systems can watch large regions
        # of memory via https://reviews.llvm.org/D149792 , don't run this
        # test there.
        if self.getArchitecture() not in ["arm64", "arm64e", "arm64_32"]:
            # Watch for write to *g_char_ptr.
            error = lldb.SBError()
            wp_opts = lldb.SBWatchpointOptions()
            wp_opts.SetWatchpointTypeWrite(lldb.eWatchpointWriteTypeOnModify)
            watchpoint = target.WatchpointCreateByAddress(
                value.GetValueAsUnsigned(), 365, wp_opts, error
            )
            self.assertFalse(watchpoint)
            self.expect(
                error.GetCString(),
                exe=False,
                substrs=["Setting one of the watchpoint resources failed"],
            )



"""
Test DIL pointer arithmetic.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestFrameVarDILGlobalVariableLookup(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def expect_var_path(self, expr, compare_to_framevar=False, value=None, type=None):
        value_dil = super().expect_var_path(expr, value=value, type=type)
        if compare_to_framevar:
            self.runCmd("settings set target.experimental.use-DIL false")
            value_frv = super().expect_var_path(expr, value=value, type=type)
            self.runCmd("settings set target.experimental.use-DIL true")
            self.assertEqual(value_dil.GetValue(), value_frv.GetValue())

    def test_dereference(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")
        self.expect_var_path("*p_int0", True, value="0")
        self.expect_var_path("*cp_int5", True, value="5")
        self.expect_var_path("*rcp_int0", True, type="const int *")
        self.expect_var_path("*offset_p", True, value="5")
        self.expect_var_path("*offset_pref", True, type="int *")
        self.expect_var_path("**pp_int0", value="0")
        self.expect_var_path("&**pp_int0", type="int *")
        self.expect_var_path("*array", value="0")
        self.expect(
            "frame var '&*p_null'",
            error=True,
            substrs=["doesn't have a valid address"],
        )
        self.expect(
            "frame var '&*p_void'",
            error=True,
            substrs=["dereference failed: cannot dereference void *: (void *) p_void"],
        )



import inspect
import os
import sys


def find_lldb_root():
    lldb_root = os.path.realpath(
        os.path.dirname(inspect.getfile(inspect.currentframe()))
    )
    while True:
        parent = os.path.dirname(lldb_root)
        if parent == lldb_root:  # dirname('/') == '/'
            raise Exception("use_lldb_suite_root.py not found")
        lldb_root = parent

        test_path = os.path.join(lldb_root, "use_lldb_suite_root.py")
        if os.path.isfile(test_path):
            return lldb_root


lldb_root = find_lldb_root()

import importlib.machinery
import importlib.util

path = os.path.join(lldb_root, "use_lldb_suite_root.py")
loader = importlib.machinery.SourceFileLoader("use_lldb_suite_root", path)
spec = importlib.util.spec_from_loader("use_lldb_suite_root", loader=loader)
module = importlib.util.module_from_spec(spec)
loader.exec_module(module)



"""
Test importing the 'std' C++ module and evaluate expressions with it.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ImportStdModule(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(macos_version=["<", "15.0"])
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        # Activate importing of std module.
        self.runCmd("settings set target.import-std-module true")
        # Calling some normal std functions that return non-template types.
        self.expect_expr("std::abs(-42)", result_type="int", result_value="42")
        self.expect_expr(
            "std::minmax<int>(1, 2).first", result_type="const int", result_value="1"
        )
        self.expect_expr("std::div(2, 1).quot", result_type="int", result_value="2")
        # Using types from std.
        self.expect_expr(
            "(std::size_t)33U", result_type="std::size_t", result_value="33"
        )
        # Calling templated functions that return non-template types.
        self.expect_expr(
            "char char_a = 'b'; char char_b = 'a'; std::swap(char_a, char_b); char_a",
            result_type="char",
            result_value="'a'",
        )

    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(macos_version=["<", "15.0"])
    def test_non_cpp_language(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        # Activate importing of std module.
        self.runCmd("settings set target.import-std-module true")
        # These languages don't support C++ modules, so they shouldn't
        # be able to evaluate the expression.
        self.expect("expr -l C -- std::minmax<int>(1, 2).first", error=True)
        self.expect("expr -l C99 -- std::minmax<int>(1, 2).first", error=True)
        self.expect("expr -l C11 -- std::minmax<int>(1, 2).first", error=True)
        self.expect("expr -l ObjC -- std::minmax<int>(1, 2).first", error=True)



"""
Test the iteration protocol for frame registers.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class RegistersIteratorTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line1 = line_number("main.cpp", "// Set break point at this line.")

    def test_iter_registers(self):
        """Test iterator works correctly for lldbutil.iter_registers()."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line1)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.LaunchProcess() failed")

        import lldbsuite.test.lldbutil as lldbutil

        for thread in process:
            if thread.GetStopReason() == lldb.eStopReasonBreakpoint:
                for frame in thread:
                    # Dump the registers of this frame using
                    # lldbutil.get_GPRs() and friends.
                    if self.TraceOn():
                        print(frame)

                    REGs = lldbutil.get_GPRs(frame)
                    num = len(REGs)
                    if self.TraceOn():
                        print("\nNumber of general purpose registers: %d" % num)
                    for reg in REGs:
                        self.assertTrue(reg)
                        if self.TraceOn():
                            print("%s => %s" % (reg.GetName(), reg.GetValue()))

                    REGs = lldbutil.get_FPRs(frame)
                    num = len(REGs)
                    if self.TraceOn():
                        print("\nNumber of floating point registers: %d" % num)
                    for reg in REGs:
                        self.assertTrue(reg)
                        if self.TraceOn():
                            print("%s => %s" % (reg.GetName(), reg.GetValue()))

                    REGs = lldbutil.get_ESRs(frame)
                    if self.platformIsDarwin():
                        if (
                            self.getArchitecture() != "armv7"
                            and self.getArchitecture() != "armv7k"
                        ):
                            num = len(REGs)
                            if self.TraceOn():
                                print("\nNumber of exception state registers: %d" % num)
                            for reg in REGs:
                                self.assertTrue(reg)
                                if self.TraceOn():
                                    print("%s => %s" % (reg.GetName(), reg.GetValue()))
                    else:
                        self.assertIsNone(REGs)

                    # And these should also work.
                    for kind in [
                        "General Purpose Registers",
                        "Floating Point Registers",
                    ]:
                        REGs = lldbutil.get_registers(frame, kind)
                        self.assertTrue(REGs)

                    REGs = lldbutil.get_registers(frame, "Exception State Registers")
                    if self.platformIsDarwin():
                        if (
                            self.getArchitecture() != "armv7"
                            and self.getArchitecture() != "armv7k"
                        ):
                            self.assertIsNotNone(REGs)
                    else:
                        self.assertIsNone(REGs)

                    # We've finished dumping the registers for frame #0.
                    break



"""
Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
See https://llvm.org/LICENSE.txt for license information.
SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""

import os
import tempfile


class OnDiskTempFile:
    def __init__(self, delete=True):
        self.path = None

    def __enter__(self):
        fd, path = tempfile.mkstemp()
        os.close(fd)
        self.path = path
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if os.path.exists(self.path):
            os.remove(self.path)



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import skipIf
from lldbsuite.test.lldbtest import line_number
import lldbdap_testcase


class TestDAP_launch_extra_launch_commands(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the "launchCommands" with extra launching settings
    """

    # Flakey on 32-bit Arm Linux.
    @skipIf(oslist=["linux"], archs=["arm$"])
    def test(self):
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")

        source = "main.c"
        first_line = line_number(source, "// breakpoint 1")
        second_line = line_number(source, "// breakpoint 2")
        # Set target binary and 2 breakpoints
        # then we can verify the "launchCommands" get run
        # also we can verify that "stopCommands" get run as the
        # breakpoints get hit
        launchCommands = [
            'target create "%s"' % (program),
            "process launch --stop-at-entry",
        ]
        initCommands = ["target list", "platform list"]
        preRunCommands = ["image list a.out", "image dump sections a.out"]
        postRunCommands = ['script print("hello world")']
        stopCommands = ["frame variable", "bt"]
        exitCommands = ["expr 2+3", "expr 3+4"]
        self.launch(
            program,
            initCommands=initCommands,
            preRunCommands=preRunCommands,
            postRunCommands=postRunCommands,
            stopCommands=stopCommands,
            exitCommands=exitCommands,
            launchCommands=launchCommands,
        )
        self.set_source_breakpoints("main.c", [first_line, second_line])

        # Get output from the console. This should contain both the
        # "initCommands" and the "preRunCommands".
        output = self.get_console()
        # Verify all "initCommands" were found in console output
        self.verify_commands("initCommands", output, initCommands)
        # Verify all "preRunCommands" were found in console output
        self.verify_commands("preRunCommands", output, preRunCommands)

        # Verify all "launchCommands" were found in console output
        # After execution, program should launch
        self.verify_commands("launchCommands", output, launchCommands)
        self.verify_commands("postRunCommands", output, postRunCommands)
        # Verify the "stopCommands" here
        self.continue_to_next_stop()
        output = self.get_console()
        self.verify_commands("stopCommands", output, stopCommands)

        # Continue and hit the second breakpoint.
        # Get output from the console. This should contain both the
        # "stopCommands" that were run after the first breakpoint was hit
        self.continue_to_next_stop()
        output = self.get_console()
        self.verify_commands("stopCommands", output, stopCommands)

        # Continue until the program exits
        self.continue_to_exit()
        # Get output from the console. This should contain both the
        # "exitCommands" that were run after the second breakpoint was hit
        output = self.get_console()
        self.verify_commands("exitCommands", output, exitCommands)



"""
Test that if you exec lldb with the stdio file handles
closed, it is able to exit without hanging.
"""


import lldb
import os
import sys
import socket

if os.name != "nt":
    import fcntl

import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *

class TestDriverWithClosedSTDIO(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    # Windows doesn't have the fcntl module, so we can't run this
    # test there.
    @skipIf(hostoslist=["windows"])
    def test_run_lldb_and_wait(self):
        """This test forks, closes the stdio channels and exec's lldb.
        Then it waits for it to exit and asserts it did that successfully"""
        pid = os.fork()
        if pid == 0:
            fcntl.fcntl(sys.stdin, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
            fcntl.fcntl(sys.stdout, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
            fcntl.fcntl(sys.stderr, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
            lldb = lldbtest_config.lldbExec
            print(f"About to run: {lldb}")
            os.execlp(
                lldb,
                lldb,
                "-x",
                "-o",
                "script print(lldb.debugger.GetNumTargets())",
                "--batch",
            )
        else:
            if pid == -1:
                print("Couldn't fork a process.")
                return
            ret_pid, status = os.waitpid(pid, 0)
            # We're really just checking that lldb doesn't stall.
            # At the time this test was written, if you close stdin
            # in an asserts build, lldb aborts.  So handle both
            # of those cases.  The failure will just be that the
            # waitpid doesn't return, and the test times out.
            self.assertFalse(os.WIFSTOPPED(status), "We either exited or crashed.")



"""
Test that when multiple threads are stopped at the same breakpoint, LLDB sends
a batched vCont with multiple step actions and only one breakpoint disable/
re-enable pair, rather than stepping each thread individually with repeated
breakpoint toggles.

Uses a mock GDB server to directly verify the packets LLDB sends.
"""

import re

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestBatchedBreakpointStepOver(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    def test(self):
        BP_ADDR = 0x0000000000401020
        # PC after stepping past the breakpoint instruction.
        STEPPED_PC = BP_ADDR + 1
        NUM_THREADS = 10
        TIDS = [0x101 + i for i in range(NUM_THREADS)]

        class MyResponder(MockGDBServerResponder):
            def __init__(self):
                MockGDBServerResponder.__init__(self)
                self.resume_count = 0
                # Track which threads have completed their step.
                self.stepped_threads = set()

            def qSupported(self, client_supported):
                return (
                    "PacketSize=3fff;QStartNoAckMode+;"
                    "qXfer:features:read+;swbreak+;hwbreak+"
                )

            def qfThreadInfo(self):
                return "m" + ",".join("{:x}".format(t) for t in TIDS)

            def qsThreadInfo(self):
                return "l"

            def haltReason(self):
                # All threads stopped at the breakpoint address.
                threads_str = ",".join("{:x}".format(t) for t in TIDS)
                pcs_str = ",".join("{:x}".format(BP_ADDR) for _ in TIDS)
                return "T05thread:{:x};threads:{};thread-pcs:{};" "swbreak:;".format(
                    TIDS[0], threads_str, pcs_str
                )

            def threadStopInfo(self, threadnum):
                threads_str = ",".join("{:x}".format(t) for t in TIDS)
                pcs_str = ",".join("{:x}".format(BP_ADDR) for _ in TIDS)
                return "T05thread:{:x};threads:{};thread-pcs:{};" "swbreak:;".format(
                    threadnum, threads_str, pcs_str
                )

            def setBreakpoint(self, packet):
                return "OK"

            def readRegisters(self):
                return "00" * 160

            def readRegister(self, regno):
                return "00" * 8

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <target version="1.0">
                          <architecture>i386:x86-64</architecture>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rax" bitsize="64" regnum="0" type="int" group="general"/>
                            <reg name="rbx" bitsize="64" regnum="1" type="int" group="general"/>
                            <reg name="rcx" bitsize="64" regnum="2" type="int" group="general"/>
                            <reg name="rdx" bitsize="64" regnum="3" type="int" group="general"/>
                            <reg name="rsi" bitsize="64" regnum="4" type="int" group="general"/>
                            <reg name="rdi" bitsize="64" regnum="5" type="int" group="general"/>
                            <reg name="rbp" bitsize="64" regnum="6" type="data_ptr" group="general"/>
                            <reg name="rsp" bitsize="64" regnum="7" type="data_ptr" group="general"/>
                            <reg name="r8" bitsize="64" regnum="8" type="int" group="general"/>
                            <reg name="r9" bitsize="64" regnum="9" type="int" group="general"/>
                            <reg name="r10" bitsize="64" regnum="10" type="int" group="general"/>
                            <reg name="r11" bitsize="64" regnum="11" type="int" group="general"/>
                            <reg name="r12" bitsize="64" regnum="12" type="int" group="general"/>
                            <reg name="r13" bitsize="64" regnum="13" type="int" group="general"/>
                            <reg name="r14" bitsize="64" regnum="14" type="int" group="general"/>
                            <reg name="r15" bitsize="64" regnum="15" type="int" group="general"/>
                            <reg name="rip" bitsize="64" regnum="16" type="code_ptr" group="general"/>
                            <reg name="eflags" bitsize="32" regnum="17" type="int" group="general"/>
                            <reg name="cs" bitsize="32" regnum="18" type="int" group="general"/>
                            <reg name="ss" bitsize="32" regnum="19" type="int" group="general"/>
                          </feature>
                        </target>""",
                        False,
                    )
                return None, False

            def other(self, packet):
                if packet == "vCont?":
                    return "vCont;c;C;s;S"
                if packet.startswith("vCont;"):
                    return self._handle_vCont(packet)
                if packet.startswith("z"):
                    return "OK"
                return ""

            def _handle_vCont(self, packet):
                self.resume_count += 1
                # Parse step actions from vCont.
                stepping_tids = []
                for action in packet[6:].split(";"):
                    if not action:
                        continue
                    if action.startswith("s:"):
                        tid_str = action[2:]
                        if "." in tid_str:
                            tid_str = tid_str.split(".")[1]
                        stepping_tids.append(int(tid_str, 16))

                # All stepping threads complete their step.
                for tid in stepping_tids:
                    self.stepped_threads.add(tid)

                all_done = self.stepped_threads >= set(TIDS)

                # Report stop, use the first stepping thread as the reporter.
                report_tid = stepping_tids[0] if stepping_tids else TIDS[0]
                threads_str = ",".join("{:x}".format(t) for t in TIDS)
                if all_done:
                    # All threads moved past breakpoint.
                    pcs_str = ",".join("{:x}".format(STEPPED_PC) for _ in TIDS)
                else:
                    # Stepped threads moved, others still at breakpoint.
                    pcs_str = ",".join(
                        "{:x}".format(
                            STEPPED_PC if t in self.stepped_threads else BP_ADDR
                        )
                        for t in TIDS
                    )
                return "T05thread:{:x};threads:{};thread-pcs:{};".format(
                    report_tid, threads_str, pcs_str
                )

        self.server.responder = MyResponder()
        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.GetNumThreads(), NUM_THREADS)

        # Set a breakpoint at BP_ADDR, all threads are already stopped there.
        bkpt = target.BreakpointCreateByAddress(BP_ADDR)
        self.assertTrue(bkpt.IsValid())

        # Continue, LLDB should step all threads over the breakpoint.
        process.Continue()

        # Collect packets from the log.
        received = self.server.responder.packetLog.get_received()

        bp_addr_hex = "{:x}".format(BP_ADDR)

        # Count z0 (disable) and Z0 (enable) packets for our breakpoint.
        z0_packets = []
        Z0_packets = []
        vcont_step_packets = []

        for pkt in received:
            if pkt.startswith("z0,{},".format(bp_addr_hex)):
                z0_packets.append(pkt)
            elif pkt.startswith("Z0,{},".format(bp_addr_hex)):
                Z0_packets.append(pkt)
            elif pkt.startswith("vCont;"):
                step_count = len(re.findall(r";s:", pkt))
                if step_count > 0:
                    vcont_step_packets.append((step_count, pkt))

        # Verify: exactly 1 breakpoint disable (z0)
        self.assertEqual(
            len(z0_packets),
            1,
            "Expected 1 z0 (disable) packet, got {}: {}".format(
                len(z0_packets), z0_packets
            ),
        )

        # The initial Z0 is the breakpoint set. After stepping, there should
        # be exactly 1 re-enable Z0 (total Z0 count = 2: set + re-enable).
        # But we set the breakpoint via SB API, so count Z0 packets with
        # our address, initial set + 1 re-enable = 2.
        self.assertEqual(
            len(Z0_packets),
            2,
            "Expected 2 Z0 packets (1 set + 1 re-enable), got {}: {}".format(
                len(Z0_packets), Z0_packets
            ),
        )

        # At least one batched vCont with multiple step actions.
        max_batch = max((count for count, _ in vcont_step_packets), default=0)
        self.assertGreaterEqual(
            max_batch,
            NUM_THREADS,
            "Expected a vCont with {} step actions (batched), "
            "but max was {}. Packets: {}".format(
                NUM_THREADS,
                max_batch,
                [(c, p) for c, p in vcont_step_packets],
            ),
        )



"""
Test lldb-dap variables request
"""

import os

import lldbdap_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


def make_buffer_verify_dict(start_idx, count, offset=0):
    verify_dict = {}
    for i in range(start_idx, start_idx + count):
        verify_dict["[%i]" % (i)] = {"type": "int", "value": str(i + offset)}
    return verify_dict


class TestDAP_variables(lldbdap_testcase.DAPTestCaseBase):
    SHARED_BUILD_TESTCASE = False

    def verify_values(self, verify_dict, actual, varref_dict=None, expression=None):
        if "equals" in verify_dict:
            verify = verify_dict["equals"]
            for key in verify:
                verify_value = verify[key]
                actual_value = actual[key]
                self.assertEqual(
                    verify_value,
                    actual_value,
                    '"%s" keys don\'t match (%s != %s) from:\n%s'
                    % (key, actual_value, verify_value, actual),
                )
        if "startswith" in verify_dict:
            verify = verify_dict["startswith"]
            for key in verify:
                verify_value = verify[key]
                actual_value = actual[key]
                startswith = actual_value.startswith(verify_value)
                self.assertTrue(
                    startswith,
                    ('"%s" value "%s" doesn\'t start with "%s")')
                    % (key, actual_value, verify_value),
                )
        if "matches" in verify_dict:
            verify = verify_dict["matches"]
            for key in verify:
                verify_value = verify[key]
                actual_value = actual[key]
                self.assertRegex(
                    actual_value,
                    verify_value,
                    ('"%s" value "%s" doesn\'t match pattern "%s")')
                    % (key, actual_value, verify_value),
                )
        if "contains" in verify_dict:
            verify = verify_dict["contains"]
            for key in verify:
                contains_array = verify[key]
                actual_value = actual[key]
                self.assertIsInstance(contains_array, list)
                for verify_value in contains_array:
                    self.assertIn(verify_value, actual_value)
        if "missing" in verify_dict:
            missing = verify_dict["missing"]
            for key in missing:
                self.assertNotIn(
                    key, actual, 'key "%s" is not expected in %s' % (key, actual)
                )
        isReadOnly = verify_dict.get("readOnly", False)
        attributes = actual.get("presentationHint", {}).get("attributes", [])
        self.assertEqual(
            isReadOnly, "readOnly" in attributes, "%s %s" % (verify_dict, actual)
        )
        hasVariablesReference = "variablesReference" in actual
        varRef = None
        if hasVariablesReference:
            # Remember variable references in case we want to test further
            # by using the evaluate name.
            varRef = actual["variablesReference"]
            if varRef != 0 and varref_dict is not None:
                if expression is None:
                    evaluateName = actual["evaluateName"]
                else:
                    evaluateName = expression
                varref_dict[evaluateName] = varRef
        if (
            "hasVariablesReference" in verify_dict
            and verify_dict["hasVariablesReference"]
        ):
            self.assertTrue(hasVariablesReference, "verify variable reference")
        if "children" in verify_dict:
            self.assertTrue(
                hasVariablesReference and varRef is not None and varRef != 0,
                ("children verify values specified for " "variable without children"),
            )

            response = self.dap_server.request_variables(varRef)
            self.verify_variables(
                verify_dict["children"], response["body"]["variables"], varref_dict
            )

    def verify_variables(self, verify_dict, variables, varref_dict=None):
        self.assertGreaterEqual(
            len(variables),
            1,
            f"No variables to verify, verify_dict={json.dumps(verify_dict, indent=4)}",
        )
        for variable in variables:
            name = variable["name"]
            if not name.startswith("std::"):
                self.assertIn(
                    name, verify_dict, 'variable "%s" in verify dictionary' % (name)
                )
                self.verify_values(verify_dict[name], variable, varref_dict)

    def darwin_dwarf_missing_obj(self, initCommands):
        self.build(debug_info="dwarf")
        program = self.getBuildArtifact("a.out")
        main_obj = self.getBuildArtifact("main.o")
        self.assertTrue(os.path.exists(main_obj))
        # Delete the main.o file that contains the debug info so we force an
        # error when we run to main and try to get variables
        os.unlink(main_obj)

        self.create_debug_adapter()
        self.assertTrue(os.path.exists(program), "executable must exist")

        self.launch(program, initCommands=initCommands)

        functions = ["main"]
        breakpoint_ids = self.set_function_breakpoints(functions)
        self.assertEqual(len(breakpoint_ids), len(functions), "expect one breakpoint")
        self.continue_to_breakpoints(breakpoint_ids)

        resp = self.dap_server.get_local_variables()
        self.assertFalse(resp["success"], "Expected to fail")
        self.assertEqual(
            f'debug map object file "{main_obj}" containing debug info does not exist, debug info will not be loaded',
            resp["body"]["error"]["format"],
        )
        self.assertTrue(resp["body"]["error"]["showUser"])

    def do_test_scopes_variables_setVariable_evaluate(
        self, enableAutoVariableSummaries: bool
    ):
        """
        Tests the "scopes", "variables", "setVariable", and "evaluate" packets.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(
            program, enableAutoVariableSummaries=enableAutoVariableSummaries
        )
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        lines = [breakpoint1_line]
        # Set breakpoint in the thread function so we can step the threads
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)
        locals = self.dap_server.get_local_variables()
        globals = self.dap_server.get_global_variables()
        buffer_children = make_buffer_verify_dict(0, 16)
        verify_locals = {
            "argc": {
                "equals": {
                    "type": "int",
                    "value": "1",
                },
            },
            "argv": {
                "equals": {"type": "const char **"},
                "startswith": {"value": "0x"},
                "hasVariablesReference": True,
            },
            "pt": {
                "equals": {
                    "type": "PointType",
                },
                "hasVariablesReference": True,
                "children": {
                    "x": {"equals": {"type": "int", "value": "11"}},
                    "y": {"equals": {"type": "int", "value": "22"}},
                    "buffer": {"children": buffer_children, "readOnly": True},
                },
                "readOnly": True,
            },
            "valid_str": {},
            "malformed_str": {},
            "x": {"equals": {"type": "int"}},
        }

        verify_globals = {
            "s_local": {"equals": {"type": "float", "value": "2.25"}},
        }
        s_global = {"equals": {"type": "int", "value": "234"}}
        g_global = {"equals": {"type": "int", "value": "123"}}
        if lldbplatformutil.getHostPlatform() == "windows":
            verify_globals["::s_global"] = s_global
            verify_globals["g_global"] = g_global
        else:
            verify_globals["s_global"] = s_global
            verify_globals["::g_global"] = g_global

        varref_dict = {}
        self.verify_variables(verify_locals, locals, varref_dict)
        self.verify_variables(verify_globals, globals, varref_dict)
        # pprint.PrettyPrinter(indent=4).pprint(varref_dict)
        # We need to test the functionality of the "variables" request as it
        # has optional parameters like "start" and "count" to limit the number
        # of variables that are fetched
        varRef = varref_dict["pt.buffer"]
        response = self.dap_server.request_variables(varRef)
        self.verify_variables(buffer_children, response["body"]["variables"])
        # Verify setting start=0 in the arguments still gets all children
        response = self.dap_server.request_variables(varRef, start=0)
        self.verify_variables(buffer_children, response["body"]["variables"])
        # Verify setting count=0 in the arguments still gets all children.
        # If count is zero, it means to get all children.
        response = self.dap_server.request_variables(varRef, count=0)
        self.verify_variables(buffer_children, response["body"]["variables"])
        # Verify setting count to a value that is too large in the arguments
        # still gets all children, and no more
        response = self.dap_server.request_variables(varRef, count=1000)
        self.verify_variables(buffer_children, response["body"]["variables"])
        # Verify setting the start index and count gets only the children we
        # want
        response = self.dap_server.request_variables(varRef, start=5, count=5)
        self.verify_variables(
            make_buffer_verify_dict(5, 5), response["body"]["variables"]
        )
        # Verify setting the start index to a value that is out of range
        # results in an empty list
        response = self.dap_server.request_variables(varRef, start=32, count=1)
        self.assertEqual(
            len(response["body"]["variables"]),
            0,
            "verify we get no variable back for invalid start",
        )

        # Test evaluate
        expressions = {
            "pt.x": {
                "equals": {"result": "11", "type": "int"},
                "hasVariablesReference": False,
            },
            "pt.buffer[2]": {
                "equals": {"result": "2", "type": "int"},
                "hasVariablesReference": False,
            },
            "pt": {
                "equals": {"type": "PointType"},
                "startswith": {
                    "result": (
                        "{x:11, y:22, buffer:{...}}"
                        if enableAutoVariableSummaries
                        else "PointType"
                    )
                },
                "hasVariablesReference": True,
            },
            "pt.buffer": {
                "equals": {"type": "int[16]"},
                "startswith": {
                    "result": (
                        "{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...}"
                        if enableAutoVariableSummaries
                        else "int[16]"
                    )
                },
                "hasVariablesReference": True,
            },
            "argv": {
                "equals": {"type": "const char **"},
                "startswith": {"result": "0x"},
                "hasVariablesReference": True,
            },
            "argv[0]": {
                "equals": {"type": "const char *"},
                "startswith": {"result": "0x"},
                "hasVariablesReference": True,
            },
            "2+3": {
                "equals": {"result": "5", "type": "int"},
                "hasVariablesReference": False,
            },
        }
        for expression in expressions:
            response = self.dap_server.request_evaluate(expression)
            self.verify_values(expressions[expression], response["body"])

        # Test setting variables
        self.set_local("argc", 123)
        argc = self.get_local_as_int("argc")
        self.assertEqual(argc, 123, "verify argc was set to 123 (123 != %i)" % (argc))

        self.set_local("argv", 0x1234)
        argv = self.get_local_as_int("argv")
        self.assertEqual(
            argv, 0x1234, "verify argv was set to 0x1234 (0x1234 != %#x)" % (argv)
        )

        # Test hexadecimal format
        response = self.set_local("argc", 42, is_hex=True)
        verify_response = {
            "type": "int",
            "value": "0x0000002a",
        }
        for key, value in verify_response.items():
            self.assertEqual(value, response["body"][key])
        self.set_local("argc", 123)

        # Set a variable value whose name is synthetic, like a variable index
        # and verify the value by reading it
        variable_value = 100
        response = self.set_variable(varRef, "[0]", variable_value)
        # Verify dap sent the correct response
        verify_response = {
            "type": "int",
            "value": str(variable_value),
        }
        for key, value in verify_response.items():
            self.assertEqual(value, response["body"][key])

        response = self.dap_server.request_variables(varRef, start=0, count=1)
        self.verify_variables(
            make_buffer_verify_dict(0, 1, variable_value), response["body"]["variables"]
        )

        # Set a variable value whose name is a real child value, like "pt.x"
        # and verify the value by reading it
        varRef = varref_dict["pt"]
        self.set_variable(varRef, "x", "g_global - 12")
        response = self.dap_server.request_variables(varRef, start=0, count=1)
        value = response["body"]["variables"][0]["value"]
        self.assertEqual(
            value, "111", "verify pt.x got set to 111 (111 != %s)" % (value)
        )

        # We check shadowed variables and that a new get_local_variables request
        # gets the right data
        breakpoint2_line = line_number(source, "// breakpoint 2")
        lines = [breakpoint2_line]
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)

        verify_locals["argc"]["equals"]["value"] = "123"
        verify_locals["pt"]["children"]["x"]["equals"]["value"] = "111"
        verify_locals["x @ main.cpp:27"] = {"equals": {"type": "int", "value": "89"}}
        verify_locals["x @ main.cpp:29"] = {"equals": {"type": "int", "value": "42"}}
        verify_locals["x @ main.cpp:31"] = {"equals": {"type": "int", "value": "72"}}

        self.verify_variables(verify_locals, self.dap_server.get_local_variables())

        # Now we verify that we correctly change the name of a variable with and without differentiator suffix
        self.assertFalse(self.set_local("x2", 9)["success"])
        self.assertFalse(self.set_local("x @ main.cpp:0", 9)["success"])

        self.assertTrue(self.set_local("x @ main.cpp:27", 19)["success"])
        self.assertTrue(self.set_local("x @ main.cpp:29", 21)["success"])
        self.assertTrue(self.set_local("x @ main.cpp:31", 23)["success"])

        # The following should have no effect
        self.assertFalse(self.set_local("x @ main.cpp:31", "invalid")["success"])

        verify_locals["x @ main.cpp:27"]["equals"]["value"] = "19"
        verify_locals["x @ main.cpp:29"]["equals"]["value"] = "21"
        verify_locals["x @ main.cpp:31"]["equals"]["value"] = "23"

        self.verify_variables(verify_locals, self.dap_server.get_local_variables())

        # The plain x variable shold refer to the innermost x
        self.assertTrue(self.set_local("x", 22)["success"])
        verify_locals["x @ main.cpp:31"]["equals"]["value"] = "22"

        self.verify_variables(verify_locals, self.dap_server.get_local_variables())

        # In breakpoint 3, there should be no shadowed variables
        breakpoint3_line = line_number(source, "// breakpoint 3")
        lines = [breakpoint3_line]
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)

        locals = self.dap_server.get_local_variables()
        names = [var["name"] for var in locals]
        # The first shadowed x shouldn't have a suffix anymore
        verify_locals["x"] = {"equals": {"type": "int", "value": "19"}}
        self.assertNotIn("x @ main.cpp:27", names)
        self.assertNotIn("x @ main.cpp:29", names)
        self.assertNotIn("x @ main.cpp:31", names)

        self.verify_variables(verify_locals, locals)

    @skipIfWindows
    def test_scopes_variables_setVariable_evaluate(self):
        self.do_test_scopes_variables_setVariable_evaluate(
            enableAutoVariableSummaries=False
        )

    @skipIfWindows
    def test_scopes_variables_setVariable_evaluate_with_descriptive_summaries(self):
        self.do_test_scopes_variables_setVariable_evaluate(
            enableAutoVariableSummaries=True
        )

    @skipIfWindows
    def do_test_scopes_and_evaluate_expansion(self, enableAutoVariableSummaries: bool):
        """
        Tests the evaluated expression expands successfully after "scopes" packets
        and permanent expressions persist.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(
            program, enableAutoVariableSummaries=enableAutoVariableSummaries
        )
        source = "main.cpp"
        lines = [
            line_number(source, "// breakpoint 1"),
            line_number(source, "// breakpoint 3"),
            line_number(source, "// breakpoint 6"),
            line_number(source, "// breakpoint 7"),
            line_number(source, "// breakpoint 8"),
        ]
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        [bp1, bp3, bp6, bp7, bp8] = breakpoint_ids
        self.continue_to_breakpoint(bp1)

        # Verify locals
        locals = self.dap_server.get_local_variables()
        buffer_children = make_buffer_verify_dict(0, 32)
        verify_locals = {
            "argc": {
                "equals": {"type": "int", "value": "1"},
                "missing": ["indexedVariables"],
            },
            "argv": {
                "equals": {"type": "const char **"},
                "startswith": {"value": "0x"},
                "hasVariablesReference": True,
                "missing": ["indexedVariables"],
            },
            "pt": {
                "equals": {"type": "PointType"},
                "hasVariablesReference": True,
                "missing": ["indexedVariables"],
                "children": {
                    "x": {
                        "equals": {"type": "int", "value": "11"},
                        "missing": ["indexedVariables"],
                    },
                    "y": {
                        "equals": {"type": "int", "value": "22"},
                        "missing": ["indexedVariables"],
                    },
                    "buffer": {
                        "children": buffer_children,
                        "equals": {"indexedVariables": 16},
                        "readOnly": True,
                    },
                },
                "readOnly": True,
            },
            "valid_str": {
                "equals": {
                    "type": "const char *",
                },
                "matches": {
                    "value": r'0x\w+ "𐌶𐌰L𐌾𐍈 C𐍈𐌼𐌴𐍃"',
                },
            },
            "malformed_str": {
                "equals": {
                    "type": "const char *",
                },
                "matches": {
                    "value": r'0x\w+ "lone trailing \\x81\\x82 bytes"',
                },
            },
            "x": {
                "equals": {"type": "int"},
                "missing": ["indexedVariables"],
            },
        }
        self.verify_variables(verify_locals, locals)

        # Evaluate expandable expression twice: once permanent (from repl)
        # the other temporary (from other UI).
        expandable_expression = {
            "name": "pt",
            "context": {
                "repl": {
                    "equals": {"type": "PointType"},
                    "equals": {
                        "result": """(PointType) $0 = {
  x = 11
  y = 22
  buffer = {
    [0] = 0
    [1] = 1
    [2] = 2
    [3] = 3
    [4] = 4
    [5] = 5
    [6] = 6
    [7] = 7
    [8] = 8
    [9] = 9
    [10] = 10
    [11] = 11
    [12] = 12
    [13] = 13
    [14] = 14
    [15] = 15
  }
}"""
                    },
                    "missing": ["indexedVariables"],
                    "hasVariablesReference": True,
                },
                "hover": {
                    "equals": {"type": "PointType"},
                    "startswith": {
                        "result": (
                            "{x:11, y:22, buffer:{...}}"
                            if enableAutoVariableSummaries
                            else "PointType"
                        )
                    },
                    "missing": ["indexedVariables"],
                    "hasVariablesReference": True,
                },
                "watch": {
                    "equals": {"type": "PointType"},
                    "startswith": {
                        "result": (
                            "{x:11, y:22, buffer:{...}}"
                            if enableAutoVariableSummaries
                            else "PointType"
                        )
                    },
                    "missing": ["indexedVariables"],
                    "hasVariablesReference": True,
                },
                "variables": {
                    "equals": {"type": "PointType"},
                    "startswith": {
                        "result": (
                            "{x:11, y:22, buffer:{...}}"
                            if enableAutoVariableSummaries
                            else "PointType"
                        )
                    },
                    "missing": ["indexedVariables"],
                    "hasVariablesReference": True,
                },
            },
            "children": {
                "x": {"equals": {"type": "int", "value": "11"}},
                "y": {"equals": {"type": "int", "value": "22"}},
                "buffer": {"children": buffer_children, "readOnly": True},
            },
        }

        # Evaluate from known contexts.
        expr_varref_dict = {}
        permanent_expandable_ref = None
        for context, verify_dict in expandable_expression["context"].items():
            response = self.dap_server.request_evaluate(
                expandable_expression["name"],
                frameIndex=0,
                threadId=None,
                context=context,
            )

            response_body = response["body"]
            if context == "repl":  # save the variablesReference
                self.assertIn("variablesReference", response_body)
                permanent_expandable_ref = response_body["variablesReference"]

            self.verify_values(
                verify_dict,
                response_body,
                expr_varref_dict,
                expandable_expression["name"],
            )

        # Evaluate locals again.
        locals = self.dap_server.get_local_variables()
        self.verify_variables(verify_locals, locals)

        # Verify the evaluated expressions before second locals evaluation
        # can be expanded.
        var_ref = expr_varref_dict[expandable_expression["name"]]
        response = self.dap_server.request_variables(var_ref)
        self.verify_variables(
            expandable_expression["children"], response["body"]["variables"]
        )

        self.continue_to_breakpoint(bp6)

        locals = self.dap_server.get_local_variables()
        self.verify_variables(
            {
                "my_var": {
                    "equals": {
                        "type": "(unnamed struct)",
                        "value": '{name:"hello world!", x:42, y:7}'
                        if enableAutoVariableSummaries
                        else "(unnamed struct)",
                        "evaluateName": "my_var",
                    },
                    "readOnly": True,
                },
            },
            locals,
        )

        self.continue_to_breakpoint(bp7)

        expr_varref_dict = {}
        locals = self.dap_server.get_local_variables()
        self.verify_variables(
            {
                "home": {
                    "equals": {
                        "type": "MySock",
                        "value": "MySock",
                        "evaluateName": "home",
                    },
                    "readOnly": True,
                },
            },
            locals,
            expr_varref_dict,
        )

        inner_var_ref = expr_varref_dict["home"]
        anon_vars = self.dap_server.request_variables(inner_var_ref)["body"][
            "variables"
        ]
        self.verify_variables(
            {
                "(anonymous)": {
                    "equals": {"type": "MySock::(anonymous union)"},
                    "matches": {
                        "value": r"{ipv4:.*, ipv6:.*}"
                        if enableAutoVariableSummaries
                        else r"MySock::\(anonymous union\)",
                    },
                    "readOnly": True,
                    "missing": ["evaluateName"],
                }
            },
            anon_vars,
        )
        inner_union_vars = self.dap_server.request_variables(
            anon_vars[0]["variablesReference"]
        )["body"]["variables"]
        self.verify_variables(
            {
                "ipv4": {
                    "equals": {
                        "type": "unsigned char[4]",
                        "evaluateName": "home.ipv4",
                    },
                    "readOnly": True,
                },
                "ipv6": {
                    "equals": {
                        "type": "unsigned char[6]",
                        "evaluateName": "home.ipv6",
                    },
                    "readOnly": True,
                },
            },
            inner_union_vars,
        )

        self.continue_to_breakpoint(bp8)

        locals = self.dap_server.get_local_variables()
        self.verify_variables(
            {
                "e": {
                    "equals": {
                        "type": "example",
                        "value": "{lo:10, hi:11}"
                        if enableAutoVariableSummaries
                        else "example",
                        "evaluateName": "e",
                    },
                    "readOnly": True,
                },
            },
            locals,
        )
        inner_bitfields_struct = self.dap_server.request_variables(
            locals[0]["variablesReference"]
        )["body"]["variables"]
        self.verify_variables(
            {
                "lo": {
                    "equals": {
                        "type": "unsigned int",
                        "value": "10",
                        "evaluateName": "e.lo",
                        "variablesReference": 0,
                    }
                },
                "(anonymous)": {
                    "equals": {
                        "type": "int",
                        "value": "0",
                        "variablesReference": 0,
                    }
                },
                "hi": {
                    "equals": {
                        "type": "unsigned int",
                        "value": "11",
                        "evaluateName": "e.hi",
                        "variablesReference": 0,
                    }
                },
            },
            inner_bitfields_struct,
        )

        # Continue to breakpoint 3, permanent variable should still exist
        # after resume.
        self.continue_to_breakpoint(bp3)

        var_ref = permanent_expandable_ref
        response = self.dap_server.request_variables(var_ref)
        self.verify_variables(
            expandable_expression["children"], response["body"]["variables"]
        )

        # Test that frame scopes have corresponding presentation hints.
        frame_id = self.dap_server.get_stackFrame()["id"]
        scopes = self.dap_server.request_scopes(frame_id)["body"]["scopes"]

        scope_names = [scope["name"] for scope in scopes]
        self.assertIn("Locals", scope_names)
        self.assertIn("Registers", scope_names)

        for scope in scopes:
            if scope["name"] == "Locals":
                self.assertEqual(scope.get("presentationHint"), "locals")
            if scope["name"] == "Registers":
                self.assertEqual(scope.get("presentationHint"), "registers")

        # Test invalid variablesReference.
        for wrong_var_ref in (-6000, -1, 4000):
            response = self.dap_server.request_variables(wrong_var_ref)
            self.assertFalse(response["success"])
            error_msg: str = response["body"]["error"]["format"]
            self.assertTrue(
                error_msg.startswith("invalid variablesReference"),
                f"seen error message : {error_msg}",
            )

    def test_scopes_and_evaluate_expansion(self):
        self.do_test_scopes_and_evaluate_expansion(enableAutoVariableSummaries=False)

    def test_scopes_and_evaluate_expansion_with_descriptive_summaries(self):
        self.do_test_scopes_and_evaluate_expansion(enableAutoVariableSummaries=True)

    def do_test_indexedVariables(self, enableSyntheticChildDebugging: bool):
        """
        Tests that arrays and lldb.SBValue objects that have synthetic child
        providers have "indexedVariables" key/value pairs. This helps the IDE
        not to fetch too many children all at once.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(
            program, enableSyntheticChildDebugging=enableSyntheticChildDebugging
        )
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 4")
        lines = [breakpoint1_line]
        # Set breakpoint in the thread function so we can step the threads
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)

        # Verify locals
        locals = self.dap_server.get_local_variables()
        # The vector variables might have one additional entry from the fake
        # "[raw]" child.
        raw_child_count = 1 if enableSyntheticChildDebugging else 0
        verify_locals = {
            "small_array": {"equals": {"indexedVariables": 5}, "readOnly": True},
            "large_array": {"equals": {"indexedVariables": 200}, "readOnly": True},
            "small_vector": {
                "equals": {"indexedVariables": 5 + raw_child_count},
                "readOnly": True,
            },
            "large_vector": {
                "equals": {"indexedVariables": 200 + raw_child_count},
                "readOnly": True,
            },
            "pt": {"missing": ["indexedVariables"], "readOnly": True},
        }
        self.verify_variables(verify_locals, locals)

        # We also verify that we produce a "[raw]" fake child with the real
        # SBValue for the synthetic type.
        verify_children = {
            "[0]": {"equals": {"type": "int", "value": "0"}},
            "[1]": {"equals": {"type": "int", "value": "0"}},
            "[2]": {"equals": {"type": "int", "value": "0"}},
            "[3]": {"equals": {"type": "int", "value": "0"}},
            "[4]": {"equals": {"type": "int", "value": "0"}},
        }
        if enableSyntheticChildDebugging:
            verify_children["[raw]"] = {
                "equals": {"type": "std::vector<int>", "value": "size=5"},
                "readOnly": True,
            }

        children = self.dap_server.request_variables(locals[2]["variablesReference"])[
            "body"
        ]["variables"]
        varref_dict = {}
        self.verify_variables(verify_children, children, varref_dict)

        if enableSyntheticChildDebugging:
            self.assertIn(
                "small_vector",
                varref_dict,
                "'evaluateName' for '[raw]' field should be the original variable name.",
            )
            raw_children = self.dap_server.request_variables(
                varref_dict["small_vector"]
            )
            self.assertTrue(
                len(raw_children["body"]["variables"]) > 0,
                "Expected std::vector to contain a raw underlying value with internal properties.",
            )

    @skipIfWindows
    def test_return_variables(self):
        """
        Test the stepping out of a function with return value show the variable correctly.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        return_name = "(Return Value)"
        verify_locals = {
            return_name: {"equals": {"type": "int", "value": "300"}, "readOnly": True},
            "argc": {},
            "argv": {},
            "pt": {"readOnly": True},
            "valid_str": {},
            "malformed_str": {},
            "x": {},
            "return_result": {"equals": {"type": "int"}},
        }

        function_name = "test_return_variable"
        breakpoint_ids = self.set_function_breakpoints([function_name])

        self.assertEqual(len(breakpoint_ids), 1)
        self.continue_to_breakpoints(breakpoint_ids)

        threads = self.dap_server.get_threads()
        for thread in threads:
            if thread.get("reason") == "breakpoint":
                thread_id = thread["id"]

                self.stepOut(threadId=thread_id)

                local_variables = self.dap_server.get_local_variables()
                varref_dict = {}

                # `verify_variable` function only checks if the local variables
                # are in the `verify_dict` passed  this will cause this test to pass
                # even if there is no return value.
                local_variable_names = [
                    variable["name"] for variable in local_variables
                ]
                self.assertIn(
                    return_name,
                    local_variable_names,
                    "return variable is not in local variables",
                )

                self.verify_variables(verify_locals, local_variables, varref_dict)
                break

        self.assertFalse(self.set_local("(Return Value)", 20)["success"])

    @skipIfWindows
    def test_indexedVariables(self):
        self.do_test_indexedVariables(enableSyntheticChildDebugging=False)

    @skipIfWindows
    def test_indexedVariables_with_raw_child_for_synthetics(self):
        self.do_test_indexedVariables(enableSyntheticChildDebugging=True)

    @skipIfWindows
    @skipIfAsan  # FIXME this fails with a non-asan issue on green dragon.
    def test_registers(self):
        """
        Test that registers whose byte size is the size of a pointer on
        the current system get formatted as lldb::eFormatAddressInfo. This
        will show the pointer value followed by a description of the address
        itself. To test this we attempt to find the PC value in the general
        purpose registers, and since we will be stopped in main.cpp, verify
        that the value for the PC starts with a pointer and is followed by
        a description that contains main.cpp.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        lines = [breakpoint1_line]
        # Set breakpoint in the thread function so we can step the threads
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)

        pc_name = None
        arch = self.getArchitecture()
        if arch == "x86_64":
            pc_name = "rip"
        elif arch == "x86":
            pc_name = "rip"
        elif arch.startswith("arm"):
            pc_name = "pc"

        if pc_name is None:
            return
        # Verify locals
        reg_sets = self.dap_server.get_registers()
        for reg_set in reg_sets:
            if reg_set["name"] == "General Purpose Registers":
                varRef = reg_set["variablesReference"]
                regs = self.dap_server.request_variables(varRef)["body"]["variables"]
                for reg in regs:
                    if reg["name"] == pc_name:
                        value = reg["value"]
                        self.assertTrue(value.startswith("0x"))
                        self.assertIn("a.out`main + ", value)
                        self.assertIn("at main.cpp:", value)

    @no_debug_info_test
    @skipUnlessDarwin
    def test_darwin_dwarf_missing_obj(self):
        """
        Test that if we build a binary with DWARF in .o files and we remove
        the .o file for main.cpp, that we get a variable named "<error>"
        whose value matches the appriopriate error. Errors when getting
        variables are returned in the LLDB API when the user should be
        notified of issues that can easily be solved by rebuilding or
        changing compiler options and are designed to give better feedback
        to the user.
        """
        self.darwin_dwarf_missing_obj(None)

    @no_debug_info_test
    @skipUnlessDarwin
    def test_darwin_dwarf_missing_obj_with_symbol_ondemand_enabled(self):
        """
        Test that if we build a binary with DWARF in .o files and we remove
        the .o file for main.cpp, that we get a variable named "<error>"
        whose value matches the appriopriate error. Test with symbol_ondemand_enabled.
        """
        initCommands = ["settings set symbols.load-on-demand true"]
        self.darwin_dwarf_missing_obj(initCommands)

    @no_debug_info_test
    @skipIfWindows
    def test_value_format(self):
        """
        Test that toggle variables value format between decimal and hexical works.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        lines = [breakpoint1_line]

        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)

        # Verify locals value format decimal
        is_hex = False
        var_pt_x = self.dap_server.get_local_variable_child("pt", "x", is_hex=is_hex)
        self.assertEqual(var_pt_x["value"], "11")
        var_pt_y = self.dap_server.get_local_variable_child("pt", "y", is_hex=is_hex)
        self.assertEqual(var_pt_y["value"], "22")

        # Verify locals value format hexical
        is_hex = True
        var_pt_x = self.dap_server.get_local_variable_child("pt", "x", is_hex=is_hex)
        self.assertEqual(var_pt_x["value"], "0x0000000b")
        var_pt_y = self.dap_server.get_local_variable_child("pt", "y", is_hex=is_hex)
        self.assertEqual(var_pt_y["value"], "0x00000016")

        # Toggle and verify locals value format decimal again
        is_hex = False
        var_pt_x = self.dap_server.get_local_variable_child("pt", "x", is_hex=is_hex)
        self.assertEqual(var_pt_x["value"], "11")
        var_pt_y = self.dap_server.get_local_variable_child("pt", "y", is_hex=is_hex)
        self.assertEqual(var_pt_y["value"], "22")

    @skipIfWindows
    def test_variable_id_uniqueness_simple(self):
        """
        Simple regression test for variable ID uniqueness across frames.
        Ensures variable IDs are not reused between different scopes/frames.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"

        bp_line = line_number(source, "// breakpoint 3")
        self.set_source_breakpoints(source, [bp_line])
        self.continue_to_next_stop()

        frames = self.get_stackFrames()
        self.assertGreaterEqual(len(frames), 2, "Need at least 2 frames")

        all_refs = set()

        for i in range(min(3, len(frames))):
            frame_id = frames[i]["id"]
            scopes = self.dap_server.request_scopes(frame_id)["body"]["scopes"]

            for scope in scopes:
                ref = scope["variablesReference"]
                if ref != 0:
                    self.assertNotIn(
                        ref, all_refs, f"Variable reference {ref} was reused!"
                    )
                    all_refs.add(ref)

        self.assertGreater(len(all_refs), 0, "Should have found variable references")
        for ref in all_refs:
            response = self.dap_server.request_variables(ref)
            self.assertTrue(response["success"], f"Failed to access reference {ref}")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestRegion(object):
    def __init__(self, start_addr, size, dirty_pages):
        self.start_addr = start_addr
        self.size = size
        self.dirty_pages = dirty_pages

    def as_packet(self):
        dirty_pages = ""
        if self.dirty_pages is not None:
            dirty_pages = (
                "dirty-pages:"
                + ",".join([format(a, "x") for a in self.dirty_pages])
                + ";"
            )
        return f"start:{self.start_addr:x};size:{self.size};permissions:r;{dirty_pages}"

    def expected_command_output(self):
        if self.dirty_pages is None:
            return [
                "Modified memory (dirty) page list provided",
                "Dirty pages:",
            ], False

        expected = [
            f"Modified memory (dirty) page list provided, {len(self.dirty_pages)} entries."
        ]
        if self.dirty_pages:
            expected.append(
                "Dirty pages: "
                + ", ".join([format(a, "#x") for a in self.dirty_pages])
                + "."
            )
        return expected, True


class TestMemoryRegionDirtyPages(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    def test(self):
        test_regions = [
            # A memory region where we don't know anything about dirty pages
            TestRegion(0, 0x100000000, None),
            # A memory region with dirty page information -- and zero dirty pages
            TestRegion(0x100000000, 4000, []),
            # A memory region with one dirty page
            TestRegion(0x100004000, 4000, [0x100004000]),
            # A memory region with multple dirty pages
            TestRegion(
                0x1000A2000,
                5000,
                [0x1000A2000, 0x1000A3000, 0x1000A4000, 0x1000A5000, 0x1000A6000],
            ),
        ]

        class MyResponder(MockGDBServerResponder):
            def qHostInfo(self):
                return "ptrsize:8;endian:little;vm-page-size:4096;"

            def qMemoryRegionInfo(self, addr):
                for region in test_regions:
                    if region.start_addr == addr:
                        return region.as_packet()

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        for test_region in test_regions:
            region = lldb.SBMemoryRegionInfo()
            err = process.GetMemoryRegionInfo(test_region.start_addr, region)
            self.assertSuccess(err)
            self.assertEqual(region.GetPageSize(), 4096)

            if test_region.dirty_pages is None:
                self.assertFalse(region.HasDirtyMemoryPageList())
                self.assertEqual(0, region.GetNumDirtyPages())
            else:
                self.assertTrue(region.HasDirtyMemoryPageList())
                self.assertEqual(
                    len(test_region.dirty_pages), region.GetNumDirtyPages()
                )

                for i, expected_dirty_page in enumerate(test_region.dirty_pages):
                    self.assertEqual(
                        expected_dirty_page, region.GetDirtyPageAddressAtIndex(i)
                    )

            substrs, matching = test_region.expected_command_output()
            self.expect(
                f"memory region 0x{test_region.start_addr:x}",
                substrs=substrs,
                matching=matching,
            )
